# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
# Copyright (c) 2012 Michael Ligh <michael.ligh@mnin.org>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

import volatility.obj as obj

class _KPCROnx86(obj.CType):
    """KPCR for 32bit windows"""

    def idt_entries(self):
        for i, entry in enumerate(self.IDT.dereference()):
            yield i, entry

    def gdt_entries(self):
        for i, entry in enumerate(self.GDT.dereference()):
            yield i, entry

    def get_kdbg(self):
        """Find this CPUs KDBG. 

        Please note the KdVersionBlock pointer is NULL on
        all KPCR structures except the one for the first CPU. 
        In some cases on x64, even the first CPU has a NULL
        KdVersionBlock, so this is really a hit-or-miss. 
        """
        DebuggerDataList = self.KdVersionBlock.dereference_as("_DBGKD_GET_VERSION64").DebuggerDataList
    
        # DebuggerDataList is a pointer to unsigned long on x86 
        # and a pointer to unsigned long long on x64. The first 
        # dereference() dereferences the pointer, and the second 
        # dereference() dereferences the unsigned long or long long
        # as the actual KDBG address. 
        return DebuggerDataList.dereference().dereference_as("_KDDEBUGGER_DATA64")

    @property
    def ProcessorBlock(self):
        return self.PrcbData

class _KPCROnx64(_KPCROnx86):
    """KPCR for x64 windows"""

    @property
    def ProcessorBlock(self):
        return self.Prcb

    @property
    def IDT(self):
        return self.IdtBase

    @property
    def GDT(self):
        return self.GdtBase 

class KPCRProfileModification(obj.ProfileModification):
    before = ['WindowsObjectClasses']

    conditions = {'os': lambda x: x == 'windows'}

    def modification(self, profile):

        if profile.metadata.get('memory_model', '32bit') == '32bit':
            kpcr_class = _KPCROnx86
        else:
            kpcr_class = _KPCROnx64

        profile.object_classes.update({'_KPCR': kpcr_class})

        profile.merge_overlay({
            '_KPRCB': [ None, { 
            'VendorString': [ None, ['String', dict(length = 13)]], 
            }]})



# Volatility
#
# Authors:
# Michael Cohen <scudette@users.sourceforge.net>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
This module implements the slow thorough process scanning

@author:       Michael Cohen
@license:      GNU General Public License 2.0 or later
@contact:      scudette@users.sourceforge.net
@organization: Volatile Systems
"""

#pylint: disable-msg=C0111

import volatility.commands as commands
import volatility.cache as cache
import volatility.utils as utils
import volatility.obj as obj
import volatility.scan as scan

class DispatchHeaderCheck(scan.ScannerCheck):
    """ A very fast check for an _EPROCESS.Pcb.Header.

    This check assumes that the type and size of
    _EPROCESS.Pcb.Header are unsigned chars, but allows their
    offsets to be determined from vtypes (so they could change
    between OS versions).
    """
    order = 10

    def __init__(self, address_space, **_kwargs):
        ## Because this checks needs to be super fast we first
        ## instantiate the _EPROCESS and work out the offsets of the
        ## type and size members. Then in the check we just read those
        ## offsets directly.
        eprocess = obj.Object("_EPROCESS", vm = address_space, offset = 0)
        self.type = eprocess.Pcb.Header.Type
        self.size = eprocess.Pcb.Header.Size
        self.buffer_size = max(self.size.obj_offset, self.type.obj_offset) + 2
        scan.ScannerCheck.__init__(self, address_space)

    def check(self, offset):
        data = self.address_space.read(offset + self.type.obj_offset, self.buffer_size)
        return data[self.type.obj_offset] == "\x03" and data[self.size.obj_offset] == "\x1b"

    def skip(self, data, offset):
        try:
            nextval = data.index("\x03", offset + 1)
            return nextval - self.type.obj_offset - offset
        except ValueError:
            ## Substring is not found - skip to the end of this data buffer
            return len(data) - offset

class CheckThreadList(scan.ScannerCheck):
    """ Checks that _EPROCESS thread list points to the kernel Address Space """
    def check(self, offset):
        eprocess = obj.Object("_EPROCESS", vm = self.address_space,
                             offset = offset)
        kernel = 0x80000000

        list_head = eprocess.ThreadListHead

        if list_head.Flink > kernel and list_head.Blink > kernel:
            return True

class CheckDTBAligned(scan.ScannerCheck):
    """ Checks that _EPROCESS.Pcb.DirectoryTableBase is aligned to 0x20 """
    def check(self, offset):
        eprocess = obj.Object("_EPROCESS", vm = self.address_space,
                             offset = offset)

        return eprocess.Pcb.DirectoryTableBase % 0x20 == 0

class CheckSynchronization(scan.ScannerCheck):
    """ Checks that _EPROCESS.WorkingSetLock and _EPROCESS.AddressCreationLock look valid """
    def check(self, offset):
        eprocess = obj.Object("_EPROCESS", vm = self.address_space,
                             offset = offset)

        event = eprocess.WorkingSetLock.Event.Header
        if event.Type != 0x1 or event.Size != 0x4:
            return False

        event = eprocess.AddressCreationLock.Event.Header
        if event.Size == 0x4 and event.Type == 0x1:
            return True

class PSDispScanner(scan.BaseScanner):
    """ This scanner carves things that look like _EPROCESS structures.

    Since the _EPROCESS does not need to be linked to the process
    list, this scanner is useful to recover terminated or cloaked
    processes.
    """
    checks = [ ("DispatchHeaderCheck", {}),
               ("CheckDTBAligned", {}),
               ("CheckThreadList", {}),
               ("CheckSynchronization", {})
               ]

class PSDispScan(commands.Command, cache.Testable):
    """ Scan Physical memory for _EPROCESS objects based on their Dispatch Headers"""

    # Declare meta information associated with this plugin

    meta_info = dict(
        author = 'Brendan Dolan-Gavitt',
        copyright = 'Copyright (c) 2007,2008 Brendan Dolan-Gavitt',
        contact = 'bdolangavitt@wesleyan.edu',
        license = 'GNU General Public License 2.0 or later',
        url = 'http://moyix.blogspot.com/',
        os = 'WIN_32_XP_SP2',
        version = '1.0',
        )

    @cache.CacheDecorator("tests/psscan")
    def calculate(self):
        address_space = utils.load_as(self._config, astype = 'physical')

        for offset in PSDispScanner().scan(address_space):
            yield obj.Object('_EPROCESS', vm = address_space, offset = offset)

    def render_dot(self, outfd, data):
        objects = set()
        links = set()

        for eprocess in data:
            label = "{0} | {1} |".format(eprocess.UniqueProcessId,
                                         eprocess.ImageFileName)
            if eprocess.ExitTime:
                label += "exited\\n{0}".format(eprocess.ExitTime)
                options = ' style = "filled" fillcolor = "lightgray" '
            else:
                label += "running"
                options = ''

            objects.add('pid{0} [label="{1}" shape="record" {2}];\n'.format(eprocess.UniqueProcessId,
                                                                            label, options))
            links.add("pid{0} -> pid{1} [];\n".format(eprocess.InheritedFromUniqueProcessId,
                                                      eprocess.UniqueProcessId))

        ## Now write the dot file
        outfd.write("digraph processtree { \ngraph [rankdir = \"TB\"];\n")
        for link in links:
            outfd.write(link)

        for item in objects:
            outfd.write(item)
        outfd.write("}")

    def render_text(self, outfd, data):
        ## Just grab the AS and scan it using our scanner
        outfd.write(" Offset     Name             PID    PPID   PDB        Time created             Time exited             \n" +
                    "---------- ---------------- ------ ------ ---------- ------------------------ ------------------------ \n")

        for eprocess in data:
            outfd.write("{0:#010x} {1:16} {2:6} {3:6} {4:#010x} {5:24} {6:24}\n".format(
                eprocess.obj_offset,
                eprocess.ImageFileName,
                eprocess.UniqueProcessId,
                eprocess.InheritedFromUniqueProcessId,
                eprocess.Pcb.DirectoryTableBase,
                eprocess.CreateTime or '',
                eprocess.ExitTime or ''))



# Volatility
# Copyright (c) 2008-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

import volatility.exceptions as exceptions
import volatility.obj as obj

pe_vtypes = {
    '_IMAGE_EXPORT_DIRECTORY': [ 0x28, {
    'Base': [ 0x10, ['unsigned int']],
    'NumberOfFunctions': [ 0x14, ['unsigned int']],
    'NumberOfNames': [ 0x18, ['unsigned int']],
    'AddressOfFunctions': [ 0x1C, ['unsigned int']],
    'AddressOfNames': [ 0x20, ['unsigned int']],
    'AddressOfNameOrdinals': [ 0x24, ['unsigned int']],
    }],
    '_IMAGE_IMPORT_DESCRIPTOR': [ 0x14, {
    # 0 for terminating null import descriptor
    'OriginalFirstThunk': [ 0x0, ['unsigned int']],
    'TimeDateStamp': [ 0x4, ['unsigned int']],
    'ForwarderChain': [ 0x8, ['unsigned int']],
    'Name': [ 0xC, ['unsigned int']],
    # If bound this has actual addresses
    'FirstThunk': [ 0x10, ['unsigned int']],
    }],
    '_IMAGE_THUNK_DATA' : [ 0x4, {
    # Fake member for testing if the highest bit is set
    'OrdinalBit' : [ 0x0, ['BitField', dict(start_bit = 31, end_bit = 32)]],
    'Function' : [ 0x0, ['pointer', ['void']]],
    'Ordinal' : [ 0x0, ['unsigned long']],
    'AddressOfData' : [ 0x0, ['unsigned int']],
    'ForwarderString' : [ 0x0, ['unsigned int']],
    }],
    '_IMAGE_IMPORT_BY_NAME' : [ None, {
    'Hint' : [ 0x0, ['unsigned short']],
    'Name' : [ 0x2, ['String', dict(length = 128)]],
    }],
}

pe_vtypes_64 = {
    '_IMAGE_THUNK_DATA' : [ 0x8, {
    # Fake member for testing if the highest bit is set
    'OrdinalBit' : [ 0x0, ['BitField', dict(start_bit = 63, end_bit = 64)]],
    'Function' : [ 0x0, ['pointer64', ['void']]],
    'Ordinal' : [ 0x0, ['unsigned long long']],
    'AddressOfData' : [ 0x0, ['unsigned long long']],
    'ForwarderString' : [ 0x0, ['unsigned long long']],
    }],
}

class _IMAGE_EXPORT_DIRECTORY(obj.CType):
    """Class for PE export directory"""

    def valid(self, nt_header):
        """
        Check the sanity of export table fields.

        The RVAs cannot be larger than the module size. The function
        and name counts cannot be larger than 32K. 
        """
        try:
            return (self.AddressOfFunctions < nt_header.OptionalHeader.SizeOfImage and
                    self.AddressOfNameOrdinals < nt_header.OptionalHeader.SizeOfImage and
                    self.AddressOfNames < nt_header.OptionalHeader.SizeOfImage and
                    self.NumberOfFunctions < 0x7FFF and
                    self.NumberOfNames < 0x7FFF)
        except obj.InvalidOffsetError:
            return False

    def _name(self, name_rva):
        """
        Return a String object for the function name.

        Names are truncated at 128 characters although its possible 
        they may be longer. Thus, infrequently a function name will
        be missing some data. However, that's better than hard-coding
        a larger value which frequently causes us to cross page 
        boundaries and return a NoneObject anyway.  
        """
        return obj.Object("String",
                      offset = self.obj_parent.DllBase + name_rva,
                      vm = self.obj_native_vm, length = 128)

    def _exported_functions(self):
        """
        Generator for exported functions.

        @return: tuple (Ordinal, FunctionRVA, Name)

        Ordinal is an integer and should never be None. If the function 
        is forwarded, FunctionRVA is None. Otherwise, FunctionRVA is an
        RVA to the function's code (relative to module base). Name is a
        String containing the exported function's name. If the Name is 
        paged, it will be None. If the function is forwarded, Name is the
        forwarded function name including the DLL (ntdll.EtwLogTraceEvent). 
        """

        mod_base = self.obj_parent.DllBase
        exp_dir = self.obj_parent.export_dir()

        # PE files with a large number of functions will have arrays
        # that spans multiple pages. Thus the first entries may be valid, 
        # last entries may be valid, but middle entries may be invalid
        # (paged). In the various checks below, we test for None (paged)
        # and zero (non-paged but invalid RVA). 

        # Array of RVAs to function code 
        address_of_functions = obj.Object('Array',
                                    offset = mod_base + self.AddressOfFunctions,
                                    targetType = 'unsigned int',
                                    count = self.NumberOfFunctions,
                                    vm = self.obj_native_vm)
        # Array of RVAs to function names 
        address_of_names = obj.Object('Array',
                                    offset = mod_base + self.AddressOfNames,
                                    targetType = 'unsigned int',
                                    count = self.NumberOfNames,
                                    vm = self.obj_native_vm)
        # Array of RVAs to function ordinals 
        address_of_name_ordinals = obj.Object('Array',
                                    offset = mod_base + self.AddressOfNameOrdinals,
                                    targetType = 'unsigned short',
                                    count = self.NumberOfNames,
                                    vm = self.obj_native_vm)

        # When functions are exported by Name, it will increase
        # NumberOfNames by 1 and NumberOfFunctions by 1. When 
        # functions are exported by Ordinal, only the NumberOfFunctions
        # will increase. First we enum functions exported by Name 
        # and track their corresponding Ordinals, so that when we enum
        # functions exported by Ordinal only, we don't duplicate. 

        seen_ordinals = []

        # Handle functions exported by name *and* ordinal 
        for i in range(self.NumberOfNames):

            name_rva = address_of_names[i]
            ordinal = address_of_name_ordinals[i]

            if name_rva in (0, None):
                continue

            # Check the sanity of ordinal values before using it as an index
            if ordinal == None or ordinal >= self.NumberOfFunctions:
                continue

            func_rva = address_of_functions[ordinal]

            if func_rva in (0, None):
                continue

            # Handle forwarded exports. If the function's RVA is inside the exports 
            # section (as given by the VirtualAddress and Size fields in the 
            # DataDirectory), the symbol is forwarded. Return the name of the 
            # forwarded function and None as the function address. 

            if (func_rva >= exp_dir.VirtualAddress and
                    func_rva < exp_dir.VirtualAddress + exp_dir.Size):
                n = self._name(func_rva)
                f = obj.NoneObject("Ordinal function {0} in module {1} forwards to {2}".format(
                                   ordinal, self.obj_parent.BaseDllName, n))
            else:
                n = self._name(name_rva)
                f = func_rva

            # Add the ordinal base and save it 
            ordinal += self.Base
            seen_ordinals.append(ordinal)

            yield ordinal, f, n

        # Handle functions exported by ordinal only 
        for i in range(self.NumberOfFunctions):

            ordinal = self.Base + i

            # Skip functions already enumberated above 
            if ordinal not in seen_ordinals:

                func_rva = address_of_functions[i]

                if func_rva in (0, None):
                    continue

                seen_ordinals.append(ordinal)

                # There is no name RVA 
                yield ordinal, func_rva, obj.NoneObject("Name RVA not accessible")

class _IMAGE_IMPORT_DESCRIPTOR(obj.CType):
    """Handles IID entries for imported functions"""

    def valid(self, nt_header):
        """Check the validity of some fields"""
        try:
            return (self.OriginalFirstThunk != 0 and
                    self.OriginalFirstThunk < nt_header.OptionalHeader.SizeOfImage and
                    self.FirstThunk != 0 and
                    self.FirstThunk < nt_header.OptionalHeader.SizeOfImage and
                    self.Name < nt_header.OptionalHeader.SizeOfImage)
        except obj.InvalidOffsetError:
            return False

    def _name(self, name_rva):
        """Return a String object for the name at the given RVA"""

        return obj.Object("String",
                      offset = self.obj_parent.DllBase + name_rva,
                      vm = self.obj_native_vm, length = 128)

    def dll_name(self):
        """Returns the name of the DLL for this IID"""
        return self._name(self.Name)

    def _imported_functions(self):
        """
        Generator for imported functions. 

        @return: tuple (Ordinal, FunctionVA, Name)

        If the function is imported by ordinal, then Ordinal is the 
        ordinal value and Name is None. 

        If the function is imported by name, then Ordinal is the
        hint and Name is the imported function name (or None if its
        paged). 

        FunctionVA is the virtual address of the imported function,
        as applied to the IAT by the Windows loader. If the FirstThunk
        is paged, then FunctionVA will be None. 
        """

        i = 0
        while 1:
            thunk = obj.Object('_IMAGE_THUNK_DATA',
                       offset = self.obj_parent.DllBase + self.OriginalFirstThunk +
                       i * self.obj_vm.profile.get_obj_size('_IMAGE_THUNK_DATA'),
                       vm = self.obj_native_vm)

            # We've reached the end when the element is zero 
            if thunk == None or thunk.AddressOfData == 0:
                break

            o = obj.NoneObject("Ordinal not accessible?")
            n = obj.NoneObject("Imported by ordinal?")
            f = obj.NoneObject("FirstThunk not accessible")

            # If the highest bit (32 for x86 and 64 for x64) is set, the function is 
            # imported by ordinal and the lowest 16-bits contain the ordinal value. 
            # Otherwise, the lowest bits (0-31 for x86 and 0-63 for x64) contain an 
            # RVA to an _IMAGE_IMPORT_BY_NAME struct. 
            if thunk.OrdinalBit == 1:
                o = thunk.Ordinal & 0xFFFF
            else:
                iibn = obj.Object("_IMAGE_IMPORT_BY_NAME",
                                  offset = self.obj_parent.DllBase +
                                  thunk.AddressOfData,
                                  vm = self.obj_native_vm)
                o = iibn.Hint
                n = iibn.Name

            # See if the import is bound (i.e. resolved)
            first_thunk = obj.Object('_IMAGE_THUNK_DATA',
                            offset = self.obj_parent.DllBase + self.FirstThunk +
                            i * self.obj_vm.profile.get_obj_size('_IMAGE_THUNK_DATA'),
                            vm = self.obj_native_vm)
            if first_thunk:
                f = first_thunk.Function.v()

            yield o, f, n
            i += 1

    def is_list_end(self):
        """Returns True if we've reached the list end"""
        data = self.obj_vm.zread(
                        self.obj_offset,
                        self.obj_vm.profile.get_obj_size('_IMAGE_IMPORT_DESCRIPTOR')
                        )
        return data.count(chr(0)) == len(data)

class _LDR_DATA_TABLE_ENTRY(obj.CType):
    """
    Class for PE file / modules

    If these classes are instantiated by _EPROCESS.list_*_modules() 
    then its guaranteed to be in the process address space. 

    FIXME: If these classes are found by modscan, ensure we can
    dereference properly with obj_native_vm. 
    """

    def _nt_header(self):
        """Return the _IMAGE_NT_HEADERS object"""

        try:
            dos_header = obj.Object("_IMAGE_DOS_HEADER", offset = self.DllBase,
                                    vm = self.obj_native_vm)

            return dos_header.get_nt_header()
        except ValueError:
            return obj.NoneObject("Failed initial sanity checks")
        except exceptions.SanityCheckException:
            return obj.NoneObject("Failed initial sanity checks. Try -u or --unsafe")

    def _directory(self, dir_index):
        """Return the requested IMAGE_DATA_DIRECTORY"""

        nt_header = self._nt_header()
        if nt_header == None:
            raise ValueError('No directory index {0}'.format(dir_index))

        data_dir = nt_header.OptionalHeader.DataDirectory[dir_index]
        if data_dir == None:
            raise ValueError('No directory index {0}'.format(dir_index))

        # Make sure the directory exists 
        if data_dir.VirtualAddress == 0 or data_dir.Size == 0:
            raise ValueError('No directory index {0}'.format(dir_index))

        # Make sure the directory VA and Size are sane 
        if data_dir.VirtualAddress + data_dir.Size > nt_header.OptionalHeader.SizeOfImage:
            raise ValueError('Invalid directory for index {0}'.format(dir_index))

        return data_dir

    def export_dir(self):
        """Return the IMAGE_DATA_DIRECTORY for exports"""
        return self._directory(0) # DIRECTORY_ENTRY_EXPORT

    def import_dir(self):
        """Return the IMAGE_DATA_DIRECTORY for imports"""
        return self._directory(1) # DIRECTORY_ENTRY_IMPORT

    def debug_dir(self):
        """Return the IMAGE_DEBUG_DIRECTORY for debug info"""
        return self._directory(6) # IMAGE_DEBUG_DIRECTORY

    def get_debug_directory(self):
        """Return the debug directory object for this PE"""
        
        try:
            data_dir = self.debug_dir()
        except ValueError, why:
            return obj.NoneObject(str(why))

        return obj.Object("_IMAGE_DEBUG_DIRECTORY", 
                          offset = self.DllBase + data_dir.VirtualAddress, 
                          vm = self.obj_native_vm)

    def getprocaddress(self, func):
        """Return the RVA of func"""
        for _, f, n in self.exports():
            if str(n or '') == func:
                return f
        return None

    def imports(self):
        """
        Generator for the PE's imported functions.

        The _DIRECTORY_ENTRY_IMPORT.VirtualAddress points to an array 
        of _IMAGE_IMPORT_DESCRIPTOR structures. The end is reached when 
        the IID structure is all zeros. 
        """

        try:
            data_dir = self.import_dir()
        except ValueError, why:
            raise StopIteration(why)

        i = 0

        desc_size = self.obj_vm.profile.get_obj_size('_IMAGE_IMPORT_DESCRIPTOR')

        while 1:
            desc = obj.Object('_IMAGE_IMPORT_DESCRIPTOR',
                      vm = self.obj_native_vm,
                      offset = self.DllBase + data_dir.VirtualAddress + (i * desc_size),
                      parent = self)

            # Stop if the IID is paged or all zeros
            if desc == None or desc.is_list_end():
                break

            # Stop if the IID contains invalid fields 
            if not desc.valid(self._nt_header()):
                break

            dll_name = desc.dll_name()

            for o, f, n in desc._imported_functions():
                yield dll_name, o, f, n

            i += 1

    def exports(self):
        """Generator for the PE's exported functions"""

        try:
            data_dir = self.export_dir()
        except ValueError, why:
            raise StopIteration(why)

        expdir = obj.Object('_IMAGE_EXPORT_DIRECTORY',
                            offset = self.DllBase + data_dir.VirtualAddress,
                            vm = self.obj_native_vm,
                            parent = self)

        if expdir.valid(self._nt_header()):
            # Ordinal, Function RVA, and Name Object 
            for o, f, n in expdir._exported_functions():
                yield o, f, n

class WinPEVTypes(obj.ProfileModification):
    before = ['WindowsOverlay']
    conditions = {'os': lambda x : x == 'windows'}
    def modification(self, profile):
        profile.vtypes.update(pe_vtypes)

class WinPEx64VTypes(obj.ProfileModification):
    before = ['WinPEVTypes']
    conditions = {'os': lambda x : x == 'windows',
                  'memory_model': lambda x: x == '64bit'}
    def modification(self, profile):
        profile.vtypes.update(pe_vtypes_64)

class WinPEObjectClasses(obj.ProfileModification):
    before = ['WindowsOverlay']
    conditions = {'os': lambda x : x == 'windows'}
    def modification(self, profile):
        profile.object_classes.update({
            '_IMAGE_EXPORT_DIRECTORY': _IMAGE_EXPORT_DIRECTORY,
            '_IMAGE_IMPORT_DESCRIPTOR': _IMAGE_IMPORT_DESCRIPTOR,
            '_LDR_DATA_TABLE_ENTRY': _LDR_DATA_TABLE_ENTRY,
            })



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""

import volatility.obj as obj
import volatility.plugins.linux.common as linux_common
import volatility.plugins.linux.pslist as linux_pslist

class linux_proc_maps(linux_pslist.linux_pslist):
    """Gathers process maps for linux"""

    def calculate(self):
        linux_common.set_plugin_members(self)
        tasks = linux_pslist.linux_pslist.calculate(self)

        for task in tasks:
            if task.mm:
                for vma in task.get_proc_maps():
                    yield task, vma            

    def render_text(self, outfd, data):
        self.table_header(outfd, [("Pid", "8"),
                                  ("Start", "#018x"),
                                  ("End",   "#018x"),
                                  ("Flags", "6"),
                                  ("Pgoff", "[addr]"),
                                  ("Major", "6"),
                                  ("Minor", "6"),
                                  ("Inode", "10"),
                                  ("File Path", "80"),                    
                                 ]) 
        for task, vma in data:

            if vma.vm_file:
                inode = vma.vm_file.dentry.d_inode
                major, minor = inode.i_sb.major, inode.i_sb.minor
                ino = inode.i_ino
                pgoff = vma.vm_pgoff << 12
                fname = linux_common.get_path(task, vma.vm_file)
            else:
                (major, minor, ino, pgoff) = [0] * 4

                if vma.vm_start <= task.mm.start_brk and vma.vm_end >= task.mm.brk:
                    fname = "[heap]"
                elif vma.vm_start <= task.mm.start_stack and vma.vm_end >= task.mm.start_stack:
                    fname = "[stack]"
                else:
                    fname = ""

            self.table_row(outfd, task.pid, 
                vma.vm_start,
                vma.vm_end,
                str(vma.vm_flags),
                pgoff,
                major,
                minor,
                ino,
                fname)



# Volatility
# Copyright (c) 2008-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

import sys
import volatility.debug as debug
import volatility.obj as obj

# SSDT structures for all x86 profiles *except* Win 2003 Server
ssdt_vtypes = {
    '_SERVICE_DESCRIPTOR_TABLE' : [ 0x40, {
    'Descriptors' : [0x0, ['array', 4, ['_SERVICE_DESCRIPTOR_ENTRY']]],
    }],
    '_SERVICE_DESCRIPTOR_ENTRY' : [ 0x10, {
    'KiServiceTable' : [0x0, ['pointer', ['void']]],
    'CounterBaseTable' : [0x4, ['pointer', ['unsigned long']]],
    'ServiceLimit' : [0x8, ['unsigned long']],
    'ArgumentTable' : [0xc, ['pointer', ['unsigned char']]],
    }],
}

# SSDT structures for Win 2003 Server x86
ssdt_vtypes_2003 = {
    '_SERVICE_DESCRIPTOR_TABLE' : [ 0x20, {
    'Descriptors' : [0x0, ['array', 2, ['_SERVICE_DESCRIPTOR_ENTRY']]],
    }],
}

# SSDT structures for x64
ssdt_vtypes_64 = {
    '_SERVICE_DESCRIPTOR_TABLE' : [ 0x40, {
    'Descriptors' : [0x0, ['array', 2, ['_SERVICE_DESCRIPTOR_ENTRY']]],
    }],
    '_SERVICE_DESCRIPTOR_ENTRY' : [ 0x20, {
    'KiServiceTable' : [0x0, ['pointer64', ['void']]],
    'CounterBaseTable' : [0x8, ['pointer64', ['unsigned long']]],
    'ServiceLimit' : [0x10, ['unsigned long long']],
    'ArgumentTable' : [0x18, ['pointer64', ['unsigned char']]],
    }],
}

#### Filthy Hack for backwards compatibility

def syscalls_property(x):
    debug.debug("Deprecation warning: Please use profile.additional['syscalls'] over profile.syscalls")
    return x.additional.get('syscalls', [[], []])

class WinSyscallsAttribute(obj.ProfileModification):
    conditions = {'os': lambda x: x == 'windows'}

    def modification(self, profile):
        # Filthy hack for backwards compatibilitiy
        profile.__class__.syscalls = property(syscalls_property)

####

class AbstractSyscalls(obj.ProfileModification):
    syscall_module = 'No default'
    def modification(self, profile):
        module = sys.modules.get(self.syscall_module, None)
        profile.additional['syscalls'] = module.syscalls

class WinXPSyscalls(AbstractSyscalls):
    syscall_module = 'volatility.plugins.overlays.windows.xp_sp2_x86_syscalls'
    conditions = {'os': lambda x: x == 'windows',
                  'memory_model': lambda x: x == '32bit',
                  'major': lambda x : x == 5,
                  'minor': lambda x : x == 1}

class Win64SyscallVTypes(obj.ProfileModification):
    before = ['WindowsVTypes']
    conditions = {'os': lambda x: x == 'windows',
                  'memory_model': lambda x: x == '64bit'}
    def modification(self, profile):
        profile.vtypes.update(ssdt_vtypes_64)

class Win2003SyscallVTypes(obj.ProfileModification):
    before = ['WindowsVTypes']
    conditions = {'os': lambda x: x == 'windows',
                  'memory_model': lambda x: x == '32bit',
                  'major': lambda x: x == 5,
                  'minor': lambda x: x == 2}
    def modification(self, profile):
        profile.vtypes.update(ssdt_vtypes_2003)

class Win2003SP0Syscalls(AbstractSyscalls):
    # Win2003SP12Syscalls applies to SP0 as well, so this must be applied second
    before = ['Win2003SP12Syscalls']
    syscall_module = 'volatility.plugins.overlays.windows.win2003_sp0_x86_syscalls'
    conditions = {'os': lambda x: x == 'windows',
                  'memory_model': lambda x: x == '32bit',
                  'major': lambda x: x == 5,
                  'minor': lambda x: x == 2,
                  'build': lambda x: x == 3789}

class Win2003SP12Syscalls(AbstractSyscalls):
    syscall_module = 'volatility.plugins.overlays.windows.win2003_sp12_x86_syscalls'
    conditions = {'os': lambda x: x == 'windows',
                  'memory_model': lambda x: x == '32bit',
                  'major': lambda x : x == 5,
                  'minor': lambda x : x == 2}

class Win2003SP12x64Syscalls(AbstractSyscalls):
    syscall_module = 'volatility.plugins.overlays.windows.win2003_sp12_x64_syscalls'
    conditions = {'os': lambda x: x == 'windows',
                  'memory_model': lambda x: x == '64bit',
                  'major': lambda x : x == 5,
                  'minor': lambda x : x == 2}

class VistaSP0Syscalls(AbstractSyscalls):
    syscall_module = 'volatility.plugins.overlays.windows.vista_sp0_x86_syscalls'
    conditions = {'os': lambda x: x == 'windows',
                  'memory_model': lambda x: x == '32bit',
                  'major': lambda x : x == 6,
                  'minor': lambda x : x == 0,
                  'build': lambda x : x == 6000}

class VistaSP0x64Syscalls(AbstractSyscalls):
    syscall_module = 'volatility.plugins.overlays.windows.vista_sp0_x64_syscalls'
    conditions = {'os': lambda x: x == 'windows',
                  'memory_model': lambda x: x == '64bit',
                  'major': lambda x : x == 6,
                  'minor': lambda x : x == 0,
                  'build': lambda x : x == 6000}

class VistaSP12Syscalls(AbstractSyscalls):
    syscall_module = 'volatility.plugins.overlays.windows.vista_sp12_x86_syscalls'
    conditions = {'os': lambda x: x == 'windows',
                  'memory_model': lambda x: x == '32bit',
                  'major': lambda x : x == 6,
                  'minor': lambda x : x == 0,
                  'build': lambda x : x >= 6001}

class VistaSP12x64Syscalls(AbstractSyscalls):
    syscall_module = 'volatility.plugins.overlays.windows.vista_sp12_x64_syscalls'
    conditions = {'os': lambda x: x == 'windows',
                  'memory_model': lambda x: x == '64bit',
                  'major': lambda x : x == 6,
                  'minor': lambda x : x == 0,
                  'build': lambda x : x >= 6001}

class Win7SP01Syscalls(AbstractSyscalls):
    syscall_module = 'volatility.plugins.overlays.windows.win7_sp01_x86_syscalls'
    conditions = {'os': lambda x: x == 'windows',
                  'memory_model': lambda x: x == '32bit',
                  'major': lambda x : x == 6,
                  'minor': lambda x : x == 1}

class Win7SP01x64Syscalls(AbstractSyscalls):
    syscall_module = 'volatility.plugins.overlays.windows.win7_sp01_x64_syscalls'
    conditions = {'os': lambda x: x == 'windows',
                  'memory_model': lambda x: x == '64bit',
                  'major': lambda x : x == 6,
                  'minor': lambda x : x == 1}



# Volatility
# Copyright (c) 2008-2013 Volatility Foundation
# Copyright (c) 2008 Brendan Dolan-Gavitt <bdolangavitt@wesleyan.edu>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

import copy
import volatility.obj as obj
import volatility.plugins.overlays.windows.windows as windows

# File-wide pylint message disable because we have a few situations where we access structs starting _
#pylint: disable-msg=W0212

class Pointer64Decorator(object):
    def __init__(self, f):
        self.f = f

    def __call__(self, name, typeList, typeDict = None):
        if len(typeList) and typeList[0] == 'pointer64':
            typeList = copy.deepcopy(typeList)
            typeList[0] = 'pointer'
        return self.f(name, typeList, typeDict)

class _EX_FAST_REF(windows._EX_FAST_REF):
    MAX_FAST_REF = 15

class ExFastRefx64(obj.ProfileModification):
    before = ['WindowsOverlay', 'WindowsObjectClasses']
    conditions = {'os': lambda x : x == 'windows',
                  'memory_model': lambda x: x == '64bit'}
    def modification(self, profile):
        profile.object_classes.update({'_EX_FAST_REF': _EX_FAST_REF})


class Windows64Overlay(obj.ProfileModification):
    before = ['WindowsOverlay', 'WindowsObjectClasses']
    conditions = {'memory_model': lambda x: x == '64bit',
                  'os': lambda x: x == 'windows'}

    def modification(self, profile):
        profile.merge_overlay({'VOLATILITY_MAGIC': [ 0x0, {
                                    'PoolAlignment': [ 0x0, ['VolatilityMagic', dict(value = 16)] ],
                                    'KUSER_SHARED_DATA': [ 0x0, ['VolatilityMagic', dict(value = 0xFFFFF78000000000)]]
                                                           }
                                                    ]})
        # This is the location of the MMVAD type which controls how to parse the
        # node. It is located before the structure.
        profile.merge_overlay({'_MMVAD_SHORT': [None, {
                                    'Tag' : [-12, None],
                                  }],
                               '_MMVAD_LONG' : [None, {
                                    'Tag' : [-12, None],
                                                       }]
                               })
        profile.vtypes["_IMAGE_NT_HEADERS"] = profile.vtypes["_IMAGE_NT_HEADERS64"]

        profile.merge_overlay({'_DBGKD_GET_VERSION64' : [  None, {
            'DebuggerDataList' : [ None, ['pointer', ['unsigned long long']]],
            }]})

        # In some auto-generated vtypes, the DTB is an array of 2 unsigned longs 
        # (for x86) or an array of 2 unsigned long long (for x64). We have an overlay
        # in windows.windows_overlay which sets the DTB to a single unsigned long,
        # but we do not want that bleeding through to the x64 profiles. Instead we 
        # want the x64 DTB to be a single unsigned long long. 
        profile.merge_overlay({'_KPROCESS' : [ None, {
            'DirectoryTableBase' : [ None, ['unsigned long long']],
            }]})

        # Note: the following method of profile modification is strongly discouraged
        #
        # Nasty hack because pointer64 has a special structure,
        # and therefore can't just be instantiated in object_classes
        # using profile.object_classes.update({'pointer64': obj.Pointer})
        profile._list_to_type = Pointer64Decorator(profile._list_to_type)



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""
import os
import volatility.obj as obj
import volatility.debug as debug
import volatility.plugins.linux.netstat as linux_netstat
import volatility.plugins.linux.common as linux_common

class linux_pkt_queues(linux_netstat.linux_netstat):
    """Writes per-process packet queues out to disk"""

    def __init__(self, config, *args, **kwargs):
        linux_netstat.linux_netstat.__init__(self, config, *args, **kwargs)
        self._config.add_option('DUMP-DIR', short_option = 'D', default = None, help = 'output directory for recovered packets', action = 'store', type = 'str')

    def process_queue(self, name, pid, fd_num, queue):
        if queue.qlen == 0:
            return

        wrote = 0

        fname = "{0:s}.{1:d}.{2:d}".format(name, pid, fd_num)
        fd = None 
 
        sk_buff = queue.m("next")

        while sk_buff and sk_buff != queue.v():

            pkt_len = sk_buff.len
            
            if pkt_len > 0 and pkt_len != 0xffffffff:

                # only open once we have a packet with data
                # otherwise we get 0 sized files
                if fd == None:
                    fd = open(os.path.join(self.edir, fname), "wb")

                start = sk_buff.data
                data  = self.addr_space.zread(start, pkt_len)

                fd.write(data)

                wrote = wrote + pkt_len
                
            sk_buff = sk_buff.next
                
        if wrote:
            yield "Wrote {0:d} bytes to {1:s}".format(wrote, fname)

        if fd:
            fd.close()

    def calculate(self):
        linux_common.set_plugin_members(self)
        self.edir = self._config.DUMP_DIR

        if not self.edir:
            debug.error("No output directory given.")

        if not os.path.isdir(self.edir):
            debug.error(self.edir + " is not a directory")

        for (task, fd_num, inet_sock) in linux_netstat.linux_netstat(self._config).calculate():

            sk = inet_sock.sk
            for msg in self.process_queue("receive", task.pid, fd_num, sk.sk_receive_queue):
                yield msg

            for msg in self.process_queue("write",   task.pid, fd_num, sk.sk_write_queue):
                yield msg

    def render_text(self, outfd, data):
        for msg in data:
            outfd.write(msg + "\n")




# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       AAron Walters and Nick Petroni
@license:      GNU General Public License 2.0
@contact:      awalters@4tphi.net, npetroni@4tphi.net
@organization: Volatility Foundation
"""

#pylint: disable-msg=C0111
import volatility.win32.tasks as tasks

def lsmod(addr_space):
    """ A Generator for modules """

    for m in tasks.get_kdbg(addr_space).modules():
        yield m



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
# Copyright (c) 2010, 2011, 2012 Michael Ligh <michael.ligh@mnin.org>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

import volatility.utils as utils
import volatility.obj as obj
import volatility.plugins.common as common
import volatility.win32.tasks as tasks
import volatility.plugins.modscan as modscan
import volatility.plugins.filescan as filescan
import volatility.plugins.overlays.windows.windows as windows
import volatility.plugins.gui.sessions as sessions
import volatility.plugins.gui.windowstations as windowstations

#--------------------------------------------------------------------------------
# object classes 
#--------------------------------------------------------------------------------

class _PSP_CID_TABLE(windows._HANDLE_TABLE): #pylint: disable-msg=W0212
    """Subclass the Windows handle table object for parsing PspCidTable"""

    def get_item(self, entry, handle_value = 0):

        p = obj.Object("address", entry.Object.v(), self.obj_vm)

        handle = obj.Object("_OBJECT_HEADER",
                offset = (p & ~7) -
                self.obj_vm.profile.get_obj_offset('_OBJECT_HEADER', 'Body'),
                vm = self.obj_vm)

        return handle

#--------------------------------------------------------------------------------
# profile modifications  
#--------------------------------------------------------------------------------

class MalwarePspCid(obj.ProfileModification):
    before = ['WindowsOverlay', 'WindowsVTypes']
    conditions = {'os': lambda x: x == 'windows'}

    def modification(self, profile):
        profile.vtypes.update({"_PSP_CID_TABLE" : profile.vtypes["_HANDLE_TABLE"]})
        profile.merge_overlay({"_KDDEBUGGER_DATA64" : [None,
            {'PspCidTable': [None,
                    ["pointer", ["pointer", ['_PSP_CID_TABLE']]]],
            }]})
        profile.object_classes.update({
            '_PSP_CID_TABLE': _PSP_CID_TABLE,
        })

#--------------------------------------------------------------------------------
# psxview plugin
#--------------------------------------------------------------------------------

class PsXview(common.AbstractWindowsCommand, sessions.SessionsMixin):
    "Find hidden processes with various process listings"

    def __init__(self, config, *args):
        common.AbstractWindowsCommand.__init__(self, config, *args)
        config.add_option("PHYSICAL-OFFSET", short_option = 'P', default = False,
                          help = "Physcal Offset", action = "store_true")

    def check_pslist(self, all_tasks):
        """Enumerate processes from PsActiveProcessHead"""
        return dict((p.obj_vm.vtop(p.obj_offset), p) for p in all_tasks)

    def check_psscan(self):
        """Enumerate processes with pool tag scanning"""
        return dict((p.obj_offset, p)
                    for p in filescan.PSScan(self._config).calculate())

    def check_thrdproc(self, _addr_space):
        """Enumerate processes indirectly by ETHREAD scanning"""
        ret = dict()

        for ethread in modscan.ThrdScan(self._config).calculate():
            if ethread.ExitTime != 0:
                continue
            # Bounce back to the threads owner 
            process = None
            if hasattr(ethread.Tcb, 'Process'):
                process = ethread.Tcb.Process.dereference_as('_EPROCESS')
            elif hasattr(ethread, 'ThreadsProcess'):
                process = ethread.ThreadsProcess.dereference()
            # Make sure the bounce succeeded 
            if (process and process.ExitTime == 0 and
                    process.UniqueProcessId > 0 and
                    process.UniqueProcessId < 65535):
                ret[process.obj_vm.vtop(process.obj_offset)] = process

        return ret

    def check_sessions(self, addr_space):
        """Enumerate processes from session structures"""
        
        ret = dict()
        for session in self.session_spaces(addr_space):
            for process in session.processes():
                ret[process.obj_vm.vtop(process.obj_offset)] = process
                
        return ret

    def check_desktop_thread(self, addr_space):
        """Enumerate processes from desktop threads"""
        
        ret = dict()
        for windowstation in windowstations.WndScan(self._config).calculate():
            for desktop in windowstation.desktops():
                for thread in desktop.threads():
                    process = thread.ppi.Process.dereference()
                    if process == None:
                        continue
                    ret[process.obj_vm.vtop(process.obj_offset)] = process
                    
        return ret

    def check_pspcid(self, addr_space):
        """Enumerate processes by walking the PspCidTable"""
        ret = dict()

        # Follow the pointers to the table base
        kdbg = tasks.get_kdbg(addr_space)
        PspCidTable = kdbg.PspCidTable.dereference().dereference()

        # Walk the handle table
        for handle in PspCidTable.handles():
            if handle.get_object_type() == "Process":
                process = handle.dereference_as("_EPROCESS")
                ret[process.obj_vm.vtop(process.obj_offset)] = process

        return ret

    def check_csrss_handles(self, all_tasks):
        """Enumerate processes using the csrss.exe handle table"""
        ret = dict()

        for p in all_tasks:
            if str(p.ImageFileName).lower() == "csrss.exe":
                # Gather the handles to process objects
                for handle in p.ObjectTable.handles():
                    if handle.get_object_type() == "Process":
                        process = handle.dereference_as("_EPROCESS")
                        ret[process.obj_vm.vtop(process.obj_offset)] = process

        return ret

    def calculate(self):
        addr_space = utils.load_as(self._config)

        all_tasks = list(tasks.pslist(addr_space))

        ps_sources = {}
        # The keys are names of process sources. The values
        # are dictionaries whose keys are physical process 
        # offsets and the values are _EPROCESS objects. 
        ps_sources['pslist'] = self.check_pslist(all_tasks)
        ps_sources['psscan'] = self.check_psscan()
        ps_sources['thrdproc'] = self.check_thrdproc(addr_space)
        ps_sources['csrss'] = self.check_csrss_handles(all_tasks)
        ps_sources['pspcid'] = self.check_pspcid(addr_space)
        ps_sources['session'] = self.check_sessions(addr_space)
        ps_sources['deskthrd'] = self.check_desktop_thread(addr_space)

        # Build a list of offsets from all sources
        seen_offsets = []
        for source in ps_sources.values():
            for offset in source.keys():
                if offset not in seen_offsets:
                    seen_offsets.append(offset)
                    yield offset, source[offset], ps_sources

    def render_text(self, outfd, data):

        self.table_header(outfd, [('Offset(P)', '[addrpad]'),
                                  ('Name', '<20'),
                                  ('PID', '>6'),
                                  ('pslist', '5'),
                                  ('psscan', '5'),
                                  ('thrdproc', '5'),
                                  ('pspcid', '5'),
                                  ('csrss', '5'),
                                  ('session', '5'),
                                  ('deskthrd', '5'),
                                  ])

        for offset, process, ps_sources in data:
            self.table_row(outfd,
                offset,
                process.ImageFileName,
                process.UniqueProcessId,
                str(ps_sources['pslist'].has_key(offset)),
                str(ps_sources['psscan'].has_key(offset)),
                str(ps_sources['thrdproc'].has_key(offset)),
                str(ps_sources['pspcid'].has_key(offset)),
                str(ps_sources['csrss'].has_key(offset)),
                str(ps_sources['session'].has_key(offset)),
                str(ps_sources['deskthrd'].has_key(offset))
                )



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""

import volatility.plugins.mac.pslist as pslist
import volatility.obj as obj
import volatility.plugins.mac.common as common

class mac_list_sessions(pslist.mac_pslist):
    """ Enumerates sessions """

    def calculate(self):
        common.set_plugin_members(self)
            
        shash_addr = self.addr_space.profile.get_symbol("_sesshash") 
        shash = obj.Object("unsigned long", offset = shash_addr, vm = self.addr_space)

        shashtbl_addr = self.addr_space.profile.get_symbol("_sesshashtbl")
        shashtbl_ptr = obj.Object("Pointer", offset = shashtbl_addr, vm = self.addr_space)
        shash_array = obj.Object(theType = "Array", targetType = "sesshashhead", count = shash + 1, vm = self.addr_space, offset = shashtbl_ptr)
    
        for sess in shash_array:
            s = sess.lh_first
    
            while s:
                yield s                
                s = s.s_hash.le_next

    def render_text(self, outfd, data):
        self.table_header(outfd, [("Leader (Pid)",  "8"),
                                  ("Leader (Name)", "20"),
                                  ("Login Name", "25")])

        for sess in data:
            if sess.s_leader:
                pid  = sess.s_leader.p_pid
                pname = sess.s_leader.p_comm
            else:
                pid = -1
                pname = "<INVALID LEADER>"
                    
            self.table_row(outfd, pid, pname, sess.s_login)




# Volatility
# Copyright (c) 2011 Michael Cohen <scudette@gmail.com>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
Support for 64 bit Linux systems.

@author:      Michael Cohen
@license:      GNU General Public License 2.0
@contact:      scudette@gmail.com
"""

from volatility import obj

class VolatilityDTB(obj.VolatilityMagic):
    """A scanner for DTB values."""

    def generate_suggestions(self):
        """Tries to locate the DTB."""
        profile = self.obj_vm.profile

        yield profile.get_symbol("init_level4_pgt") - 0xffffffff80000000

class Linux64ObjectClasses(obj.ProfileModification):
    """ Makes slight changes to the DTB checker """
    conditions = {'os': lambda x: x == 'linux',
                  'memory_model': lambda x: x == '64bit'}
    before = ['LinuxObjectClasses']
    def modification(self, profile):
        profile.object_classes.update({
            'VolatilityDTB': VolatilityDTB
                                       })



# Volatility
#
# Authors:
# Mike Auty <mike.auty@gmail.com>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

#pylint: disable-msg=C0111

import volatility.plugins.common as common
import volatility.debug as debug
import volatility.win32 as win32
import volatility.utils as utils
import volatility.protos as protos

class Sockets(common.AbstractWindowsCommand):
    """Print list of open sockets"""
    def __init__(self, config, *args, **kwargs):
        common.AbstractWindowsCommand.__init__(self, config, *args, **kwargs)
        config.add_option("PHYSICAL-OFFSET", short_option = 'P', default = False,
                          cache_invalidator = False,
                          help = "Physical Offset", action = "store_true")

    @staticmethod
    def is_valid_profile(profile):
        return (profile.metadata.get('os', 'unknown') == 'windows' and
                profile.metadata.get('major', 0) == 5)

    def render_text(self, outfd, data):
        offsettype = "(V)" if not self._config.PHYSICAL_OFFSET else "(P)"
        self.table_header(outfd,
                          [("Offset{0}".format(offsettype), "[addrpad]"),
                           ("PID", ">8"),
                           ("Port", ">6"),
                           ("Proto", ">6"),
                           ("Protocol", "15"),
                           ("Address", "15"),
                           ("Create Time", "")
                           ])

        for sock in data:
            if not self._config.PHYSICAL_OFFSET:
                offset = sock.obj_offset
            else:
                offset = sock.obj_vm.vtop(sock.obj_offset)

            self.table_row(outfd, offset, sock.Pid, sock.LocalPort, sock.Protocol,
                           protos.protos.get(sock.Protocol.v(), "-"),
                           sock.LocalIpAddress, sock.CreateTime)

    def calculate(self):
        addr_space = utils.load_as(self._config)

        if not self.is_valid_profile(addr_space.profile):
            debug.error("This command does not support the selected profile.")

        return win32.network.determine_sockets(addr_space)



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       AAron Walters
@license:      GNU General Public License 2.0
@contact:      awalters@4tphi.net
@organization: Volatility Foundation
"""

#pylint: disable-msg=C0111

import volatility.win32 as win32
import volatility.obj as obj

module_versions_xp = {
'MP' : {
  'TCBTableOff' : [0x497e8],
  'SizeOff' : [0x3f7c8],
  'AddrObjTableOffset' : [0x48760],
  'AddrObjTableSizeOffset' : [0x48764],
},
'UP' : {
  'TCBTableOff' : [0x495e8],
  'SizeOff' : [0x3f5bc],
  'AddrObjTableOffset' : [0x48560],
  'AddrObjTableSizeOffset' : [0x48564],
},
'2180' : {
  'TCBTableOff' : [0x493e8],
  'SizeOff' : [0x3f3b0],
  'AddrObjTableOffset'  : [0x48360],
  'AddrObjTableSizeOffset' : [0x48364],
},
'3244' : {
  'TCBTableOff' : [0x496E8],
  'SizeOff' : [0x3F6BC],
  'AddrObjTableOffset'  : [0x48660],
  'AddrObjTableSizeOffset' : [0x48664],
},
'3394': {
  'TCBTableOff': [0x49768],
  'SizeOff': [0x3F73C],
  'AddrObjTableOffset': [0x486E0],
  'AddrObjTableSizeOffset': [0x486E4],
},
'5625' : {
  'TCBTableOff' : [0x49ae8],
  'SizeOff' : [0x3fac8],
  'AddrObjTableOffset'  : [0x48a60],
  'AddrObjTableSizeOffset' : [0x48a64],
},
'2111' : {
  'TCBTableOff' : [0x49A68],
  'SizeOff' : [0x3FA48],
  'AddrObjTableOffset'  : [0x489E0],
  'AddrObjTableSizeOffset' : [0x489E4],
},
}

module_versions_2003 = {
# w2003 sp0
'3790' : {
 'TCBTableOff' : [0x4c6c8],
 'SizeOff' : [0x4312c],
 'AddrObjTableOffset'  : [0x4bba0],
 'AddrObjTableSizeOffset' : [0x4bba4],
},
# w2003 sp1
'1830' : {
 'TCBTableOff' : [0x4e428],
 'SizeOff' : [0x44140],
 'AddrObjTableOffset'  : [0x4d4e4],
 'AddrObjTableSizeOffset' : [0x4d4e8],
},
# w2003 sp2
'3959' : {
 'TCBTableOff' : [0x7c548],
 'SizeOff' : [0x50308],
 'AddrObjTableOffset'  : [0x5ada4],
 'AddrObjTableSizeOffset' : [0x5ada8],
},
# w2003 sp2
'4573' : {
 'TCBTableOff' : [0x7f0ac],
 'SizeOff' : [0x52328],
 'AddrObjTableOffset'  : [0x5cf04],
 'AddrObjTableSizeOffset' : [0x5cf08],
},
# w2003 sp2 x64
'3959_x64' : { 
 'TCBTableOff' : [0x000c8d30],
 'SizeOff' : [0x0009b4a0],
 'AddrObjTableOffset'  : [0x000a4880],
 'AddrObjTableSizeOffset' : [0x000a4888],
},
# w2003 sp1 x64
'1830_x64' : { 
 'TCBTableOff' : [0x8f2d0],
 'SizeOff' : [0x861cc],
 'AddrObjTableOffset'  : [0x8c4c0],
 'AddrObjTableSizeOffset' : [0x8c4c8],
},
# w2003 sp2 x64 (unknown build number)
'unk_1_x64' : { 
 'TCBTableOff' : [0xCD2D8],
 'SizeOff' : [0x9E4A0],
 'AddrObjTableOffset'  : [0xa78E0],
 'AddrObjTableSizeOffset' : [0xa78E8],
},
}
 
## Define the maxiumum number of sockets that we expect to see on a given system. 
## Due to the way we currently iterate over possible offsets, its easy to pick 
## the wrong one and end up creating an array of up to 0xFFFFFFFF objects, even 
## though there's no possibility of ever having that many active at one time. 
## This can lead to a MemoryError, which is bad. The limit we've chosen (2 million) 
## is based on 65535 for TCP, 65535 for UDP, for each of up to 100 IP addresses;
## then rounded up to the nearest million. Its not perfect, but it should prevent
## memory errors until we redesign the way we find socket and connection objects.
MAX_SOCKETS = 2000000

def determine_connections(addr_space):
    """Determines all connections for each module"""
    all_modules = win32.modules.lsmod(addr_space)

    version = (addr_space.profile.metadata.get('major', 0),
               addr_space.profile.metadata.get('minor', 0))

    if version <= (5, 1):
        module_versions = module_versions_xp
    else:
        module_versions = module_versions_2003

    for m in all_modules:
        if str(m.BaseDllName).lower() == 'tcpip.sys':
            for attempt in module_versions:
                table_size = obj.Object(
                    "long",
                    offset = m.DllBase +
                             module_versions[attempt]['SizeOff'][0],
                    vm = addr_space)

                table_addr = obj.Object(
                    "address",
                    offset = m.DllBase +
                             module_versions[attempt]['TCBTableOff'][0],
                    vm = addr_space)

                if table_size > 0:
                    table = obj.Object("Array",
                        offset = table_addr, vm = addr_space,
                        count = table_size,
                        target = obj.Curry(obj.Pointer, '_TCPT_OBJECT'))

                    if table:
                        for entry in table:
                            conn = entry.dereference()
                            seen = set()
                            while conn.is_valid() and conn.obj_offset not in seen:
                                yield conn
                                seen.add(conn.obj_offset)
                                conn = conn.Next.dereference()

def determine_sockets(addr_space):
    """Determines all sockets for each module"""
    all_modules = win32.modules.lsmod(addr_space)

    if addr_space.profile.metadata.get('major', 0) <= 5.1 and addr_space.profile.metadata.get('minor', 0) == 1:
        module_versions = module_versions_xp
    else:
        module_versions = module_versions_2003

    for m in all_modules:
        if str(m.BaseDllName).lower() == 'tcpip.sys':
            for attempt in module_versions:
                table_size = obj.Object(
                    "unsigned long",
                    offset = m.DllBase +
                             module_versions[attempt]['AddrObjTableSizeOffset'][0],
                    vm = addr_space)

                table_addr = obj.Object(
                    "address",
                    offset = m.DllBase +
                             module_versions[attempt]['AddrObjTableOffset'][0],
                    vm = addr_space)

                if int(table_size) > 0 and int(table_size) < MAX_SOCKETS:
                    table = obj.Object("Array",
                        offset = table_addr, vm = addr_space,
                        count = table_size,
                        target = obj.Curry(obj.Pointer, "_ADDRESS_OBJECT"))

                    if table:
                        for entry in table:
                            sock = entry.dereference()
                            seen = set()
                            while sock.is_valid() and sock.obj_offset not in seen:
                                yield sock
                                seen.add(sock.obj_offset)
                                sock = sock.Next.dereference()



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""

import volatility.plugins.linux.pslist as linux_pslist

class linux_psaux(linux_pslist.linux_pslist):
    '''Gathers processes along with full command line and start time'''

    def render_text(self, outfd, data):

        outfd.write("{1:6s} {2:6s} {3:6s} {0:64s}\n".format("Arguments", "Pid", "Uid", "Gid"))

        for task in data:
            outfd.write("{1:6s} {2:6s} {3:6s} {0:64s}\n".format(task.get_commandline(), str(task.pid), str(task.uid), str(task.gid)))



# Volatility
# Copyright (c) 2008-2013 Volatility Foundation
# Copyright (c) 2011 Michael Hale Ligh <michael.hale@gmail.com>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

syscalls = [
    [
    'NtAcceptConnectPort', # 0x0
    'NtAccessCheck', # 0x1
    'NtAccessCheckAndAuditAlarm', # 0x2
    'NtAccessCheckByType', # 0x3
    'NtAccessCheckByTypeAndAuditAlarm', # 0x4
    'NtAccessCheckByTypeResultList', # 0x5
    'NtAccessCheckByTypeResultListAndAuditAlarm', # 0x6
    'NtAccessCheckByTypeResultListAndAuditAlarmByHandle', # 0x7
    'NtAddAtom', # 0x8
    'NtAddBootEntry', # 0x9
    'NtAdjustGroupsToken', # 0xa
    'NtAdjustPrivilegesToken', # 0xb
    'NtAlertResumeThread', # 0xc
    'NtAlertThread', # 0xd
    'NtAllocateLocallyUniqueId', # 0xe
    'NtAllocateUserPhysicalPages', # 0xf
    'NtAllocateUuids', # 0x10
    'NtAllocateVirtualMemory', # 0x11
    'NtAreMappedFilesTheSame', # 0x12
    'NtAssignProcessToJobObject', # 0x13
    'NtCallbackReturn', # 0x14
    'NtCancelDeviceWakeupRequest', # 0x15
    'NtCancelIoFile', # 0x16
    'NtCancelTimer', # 0x17
    'NtClearEvent', # 0x18
    'NtClose', # 0x19
    'NtCloseObjectAuditAlarm', # 0x1a
    'NtCompactKeys', # 0x1b
    'NtCompareTokens', # 0x1c
    'NtCompleteConnectPort', # 0x1d
    'NtCompressKey', # 0x1e
    'NtConnectPort', # 0x1f
    'NtContinue', # 0x20
    'NtCreateDebugObject', # 0x21
    'NtCreateDirectoryObject', # 0x22
    'NtCreateEvent', # 0x23
    'NtCreateEventPair', # 0x24
    'NtCreateFile', # 0x25
    'NtCreateIoCompletion', # 0x26
    'NtCreateJobObject', # 0x27
    'NtCreateJobSet', # 0x28
    'NtCreateKey', # 0x29
    'NtCreateMailslotFile', # 0x2a
    'NtCreateMutant', # 0x2b
    'NtCreateNamedPipeFile', # 0x2c
    'NtCreatePagingFile', # 0x2d
    'NtCreatePort', # 0x2e
    'NtCreateProcess', # 0x2f
    'NtCreateProcessEx', # 0x30
    'NtCreateProfile', # 0x31
    'NtCreateSection', # 0x32
    'NtCreateSemaphore', # 0x33
    'NtCreateSymbolicLinkObject', # 0x34
    'NtCreateThread', # 0x35
    'NtCreateTimer', # 0x36
    'NtCreateToken', # 0x37
    'NtCreateWaitablePort', # 0x38
    'NtDebugActiveProcess', # 0x39
    'NtDebugContinue', # 0x3a
    'NtDelayExecution', # 0x3b
    'NtDeleteAtom', # 0x3c
    'NtDeleteBootEntry', # 0x3d
    'NtDeleteFile', # 0x3e
    'NtDeleteKey', # 0x3f
    'NtDeleteObjectAuditAlarm', # 0x40
    'NtDeleteValueKey', # 0x41
    'NtDeviceIoControlFile', # 0x42
    'NtDisplayString', # 0x43
    'NtDuplicateObject', # 0x44
    'NtDuplicateToken', # 0x45
    'NtEnumerateBootEntries', # 0x46
    'NtEnumerateKey', # 0x47
    'NtEnumerateSystemEnvironmentValuesEx', # 0x48
    'NtEnumerateValueKey', # 0x49
    'NtExtendSection', # 0x4a
    'NtFilterToken', # 0x4b
    'NtFindAtom', # 0x4c
    'NtFlushBuffersFile', # 0x4d
    'NtFlushInstructionCache', # 0x4e
    'NtFlushKey', # 0x4f
    'NtFlushVirtualMemory', # 0x50
    'NtFlushWriteBuffer', # 0x51
    'NtFreeUserPhysicalPages', # 0x52
    'NtFreeVirtualMemory', # 0x53
    'NtFsControlFile', # 0x54
    'NtGetContextThread', # 0x55
    'NtGetDevicePowerState', # 0x56
    'NtGetPlugPlayEvent', # 0x57
    'NtGetWriteWatch', # 0x58
    'NtImpersonateAnonymousToken', # 0x59
    'NtImpersonateClientOfPort', # 0x5a
    'NtImpersonateThread', # 0x5b
    'NtInitializeRegistry', # 0x5c
    'NtInitiatePowerAction', # 0x5d
    'NtIsProcessInJob', # 0x5e
    'NtIsSystemResumeAutomatic', # 0x5f
    'NtListenPort', # 0x60
    'NtLoadDriver', # 0x61
    'NtLoadKey', # 0x62
    'NtLoadKey2', # 0x63
    'NtLockFile', # 0x64
    'NtLockProductActivationKeys', # 0x65
    'NtLockRegistryKey', # 0x66
    'NtLockVirtualMemory', # 0x67
    'NtMakePermanentObject', # 0x68
    'NtMakeTemporaryObject', # 0x69
    'NtMapUserPhysicalPages', # 0x6a
    'NtMapUserPhysicalPagesScatter', # 0x6b
    'NtMapViewOfSection', # 0x6c
    'NtModifyBootEntry', # 0x6d
    'NtNotifyChangeDirectoryFile', # 0x6e
    'NtNotifyChangeKey', # 0x6f
    'NtNotifyChangeMultipleKeys', # 0x70
    'NtOpenDirectoryObject', # 0x71
    'NtOpenEvent', # 0x72
    'NtOpenEventPair', # 0x73
    'NtOpenFile', # 0x74
    'NtOpenIoCompletion', # 0x75
    'NtOpenJobObject', # 0x76
    'NtOpenKey', # 0x77
    'NtOpenMutant', # 0x78
    'NtOpenObjectAuditAlarm', # 0x79
    'NtOpenProcess', # 0x7a
    'NtOpenProcessToken', # 0x7b
    'NtOpenProcessTokenEx', # 0x7c
    'NtOpenSection', # 0x7d
    'NtOpenSemaphore', # 0x7e
    'NtOpenSymbolicLinkObject', # 0x7f
    'NtOpenThread', # 0x80
    'NtOpenThreadToken', # 0x81
    'NtOpenThreadTokenEx', # 0x82
    'NtOpenTimer', # 0x83
    'NtPlugPlayControl', # 0x84
    'NtPowerInformation', # 0x85
    'NtPrivilegeCheck', # 0x86
    'NtPrivilegeObjectAuditAlarm', # 0x87
    'NtPrivilegedServiceAuditAlarm', # 0x88
    'NtProtectVirtualMemory', # 0x89
    'NtPulseEvent', # 0x8a
    'NtQueryAttributesFile', # 0x8b
    'NtQueryBootEntryOrder', # 0x8c
    'NtQueryBootOptions', # 0x8d
    'NtQueryDebugFilterState', # 0x8e
    'NtQueryDefaultLocale', # 0x8f
    'NtQueryDefaultUILanguage', # 0x90
    'NtQueryDirectoryFile', # 0x91
    'NtQueryDirectoryObject', # 0x92
    'NtQueryEaFile', # 0x93
    'NtQueryEvent', # 0x94
    'NtQueryFullAttributesFile', # 0x95
    'NtQueryInformationAtom', # 0x96
    'NtQueryInformationFile', # 0x97
    'NtQueryInformationJobObject', # 0x98
    'NtQueryInformationPort', # 0x99
    'NtQueryInformationProcess', # 0x9a
    'NtQueryInformationThread', # 0x9b
    'NtQueryInformationToken', # 0x9c
    'NtQueryInstallUILanguage', # 0x9d
    'NtQueryIntervalProfile', # 0x9e
    'NtQueryIoCompletion', # 0x9f
    'NtQueryKey', # 0xa0
    'NtQueryMultipleValueKey', # 0xa1
    'NtQueryMutant', # 0xa2
    'NtQueryObject', # 0xa3
    'NtQueryOpenSubKeys', # 0xa4
    'NtQueryPerformanceCounter', # 0xa5
    'NtQueryQuotaInformationFile', # 0xa6
    'NtQuerySection', # 0xa7
    'NtQuerySecurityObject', # 0xa8
    'NtQuerySemaphore', # 0xa9
    'NtQuerySymbolicLinkObject', # 0xaa
    'NtQuerySystemEnvironmentValue', # 0xab
    'NtQuerySystemEnvironmentValueEx', # 0xac
    'NtQuerySystemInformation', # 0xad
    'NtQuerySystemTime', # 0xae
    'NtQueryTimer', # 0xaf
    'NtQueryTimerResolution', # 0xb0
    'NtQueryValueKey', # 0xb1
    'NtQueryVirtualMemory', # 0xb2
    'NtQueryVolumeInformationFile', # 0xb3
    'NtQueueApcThread', # 0xb4
    'NtRaiseException', # 0xb5
    'NtRaiseHardError', # 0xb6
    'NtReadFile', # 0xb7
    'NtReadFileScatter', # 0xb8
    'NtReadRequestData', # 0xb9
    'NtReadVirtualMemory', # 0xba
    'NtRegisterThreadTerminatePort', # 0xbb
    'NtReleaseMutant', # 0xbc
    'NtReleaseSemaphore', # 0xbd
    'NtRemoveIoCompletion', # 0xbe
    'NtRemoveProcessDebug', # 0xbf
    'NtRenameKey', # 0xc0
    'NtReplaceKey', # 0xc1
    'NtReplyPort', # 0xc2
    'NtReplyWaitReceivePort', # 0xc3
    'NtReplyWaitReceivePortEx', # 0xc4
    'NtReplyWaitReplyPort', # 0xc5
    'NtRequestDeviceWakeup', # 0xc6
    'NtRequestPort', # 0xc7
    'NtRequestWaitReplyPort', # 0xc8
    'NtRequestWakeupLatency', # 0xc9
    'NtResetEvent', # 0xca
    'NtResetWriteWatch', # 0xcb
    'NtRestoreKey', # 0xcc
    'NtResumeProcess', # 0xcd
    'NtResumeThread', # 0xce
    'NtSaveKey', # 0xcf
    'NtSaveKeyEx', # 0xd0
    'NtSaveMergedKeys', # 0xd1
    'NtSecureConnectPort', # 0xd2
    'NtSetBootEntryOrder', # 0xd3
    'NtSetBootOptions', # 0xd4
    'NtSetContextThread', # 0xd5
    'NtSetDebugFilterState', # 0xd6
    'NtSetDefaultHardErrorPort', # 0xd7
    'NtSetDefaultLocale', # 0xd8
    'NtSetDefaultUILanguage', # 0xd9
    'NtSetEaFile', # 0xda
    'NtSetEvent', # 0xdb
    'NtSetEventBoostPriority', # 0xdc
    'NtSetHighEventPair', # 0xdd
    'NtSetHighWaitLowEventPair', # 0xde
    'NtSetInformationDebugObject', # 0xdf
    'NtSetInformationFile', # 0xe0
    'NtSetInformationJobObject', # 0xe1
    'NtSetInformationKey', # 0xe2
    'NtSetInformationObject', # 0xe3
    'NtSetInformationProcess', # 0xe4
    'NtSetInformationThread', # 0xe5
    'NtSetInformationToken', # 0xe6
    'NtSetIntervalProfile', # 0xe7
    'NtSetIoCompletion', # 0xe8
    'NtSetLdtEntries', # 0xe9
    'NtSetLowEventPair', # 0xea
    'NtSetLowWaitHighEventPair', # 0xeb
    'NtSetQuotaInformationFile', # 0xec
    'NtSetSecurityObject', # 0xed
    'NtSetSystemEnvironmentValue', # 0xee
    'NtSetSystemEnvironmentValueEx', # 0xef
    'NtSetSystemInformation', # 0xf0
    'NtSetSystemPowerState', # 0xf1
    'NtSetSystemTime', # 0xf2
    'NtSetThreadExecutionState', # 0xf3
    'NtSetTimer', # 0xf4
    'NtSetTimerResolution', # 0xf5
    'NtSetUuidSeed', # 0xf6
    'NtSetValueKey', # 0xf7
    'NtSetVolumeInformationFile', # 0xf8
    'NtShutdownSystem', # 0xf9
    'NtSignalAndWaitForSingleObject', # 0xfa
    'NtStartProfile', # 0xfb
    'NtStopProfile', # 0xfc
    'NtSuspendProcess', # 0xfd
    'NtSuspendThread', # 0xfe
    'NtSystemDebugControl', # 0xff
    'NtTerminateJobObject', # 0x100
    'NtTerminateProcess', # 0x101
    'NtTerminateThread', # 0x102
    'NtTestAlert', # 0x103
    'NtTraceEvent', # 0x104
    'NtTranslateFilePath', # 0x105
    'NtUnloadDriver', # 0x106
    'NtUnloadKey', # 0x107
    'NtUnloadKeyEx', # 0x108
    'NtUnlockFile', # 0x109
    'NtUnlockVirtualMemory', # 0x10a
    'NtUnmapViewOfSection', # 0x10b
    'NtVdmControl', # 0x10c
    'NtWaitForDebugEvent', # 0x10d
    'NtWaitForMultipleObjects', # 0x10e
    'NtWaitForSingleObject', # 0x10f
    'NtWaitHighEventPair', # 0x110
    'NtWaitLowEventPair', # 0x111
    'NtWriteFile', # 0x112
    'NtWriteFileGather', # 0x113
    'NtWriteRequestData', # 0x114
    'NtWriteVirtualMemory', # 0x115
    'NtYieldExecution', # 0x116
    'NtCreateKeyedEvent', # 0x117
    'NtOpenKeyedEvent', # 0x118
    'NtReleaseKeyedEvent', # 0x119
    'NtWaitForKeyedEvent', # 0x11a
    'NtQueryPortInformationProcess', # 0x11b
    ],
    [
    'NtGdiAbortDoc', # 0x0
    'NtGdiAbortPath', # 0x1
    'NtGdiAddFontResourceW', # 0x2
    'NtGdiAddRemoteFontToDC', # 0x3
    'NtGdiAddFontMemResourceEx', # 0x4
    'NtGdiRemoveMergeFont', # 0x5
    'NtGdiAddRemoteMMInstanceToDC', # 0x6
    'NtGdiAlphaBlend', # 0x7
    'NtGdiAngleArc', # 0x8
    'NtGdiAnyLinkedFonts', # 0x9
    'NtGdiFontIsLinked', # 0xa
    'NtGdiArcInternal', # 0xb
    'NtGdiBeginPath', # 0xc
    'NtGdiBitBlt', # 0xd
    'NtGdiCancelDC', # 0xe
    'NtGdiCheckBitmapBits', # 0xf
    'NtGdiCloseFigure', # 0x10
    'NtGdiClearBitmapAttributes', # 0x11
    'NtGdiClearBrushAttributes', # 0x12
    'NtGdiColorCorrectPalette', # 0x13
    'NtGdiCombineRgn', # 0x14
    'NtGdiCombineTransform', # 0x15
    'NtGdiComputeXformCoefficients', # 0x16
    'NtGdiConsoleTextOut', # 0x17
    'NtGdiConvertMetafileRect', # 0x18
    'NtGdiCreateBitmap', # 0x19
    'NtGdiCreateClientObj', # 0x1a
    'NtGdiCreateColorSpace', # 0x1b
    'NtGdiCreateColorTransform', # 0x1c
    'NtGdiCreateCompatibleBitmap', # 0x1d
    'NtGdiCreateCompatibleDC', # 0x1e
    'NtGdiCreateDIBBrush', # 0x1f
    'NtGdiCreateDIBitmapInternal', # 0x20
    'NtGdiCreateDIBSection', # 0x21
    'NtGdiCreateEllipticRgn', # 0x22
    'NtGdiCreateHalftonePalette', # 0x23
    'NtGdiCreateHatchBrushInternal', # 0x24
    'NtGdiCreateMetafileDC', # 0x25
    'NtGdiCreatePaletteInternal', # 0x26
    'NtGdiCreatePatternBrushInternal', # 0x27
    'NtGdiCreatePen', # 0x28
    'NtGdiCreateRectRgn', # 0x29
    'NtGdiCreateRoundRectRgn', # 0x2a
    'NtGdiCreateServerMetaFile', # 0x2b
    'NtGdiCreateSolidBrush', # 0x2c
    'NtGdiD3dContextCreate', # 0x2d
    'NtGdiD3dContextDestroy', # 0x2e
    'NtGdiD3dContextDestroyAll', # 0x2f
    'NtGdiD3dValidateTextureStageState', # 0x30
    'NtGdiD3dDrawPrimitives2', # 0x31
    'NtGdiDdGetDriverState', # 0x32
    'NtGdiDdAddAttachedSurface', # 0x33
    'NtGdiDdAlphaBlt', # 0x34
    'NtGdiDdAttachSurface', # 0x35
    'NtGdiDdBeginMoCompFrame', # 0x36
    'NtGdiDdBlt', # 0x37
    'NtGdiDdCanCreateSurface', # 0x38
    'NtGdiDdCanCreateD3DBuffer', # 0x39
    'NtGdiDdColorControl', # 0x3a
    'NtGdiDdCreateDirectDrawObject', # 0x3b
    'NtGdiDdCreateSurface', # 0x3c
    'NtGdiDdCreateD3DBuffer', # 0x3d
    'NtGdiDdCreateMoComp', # 0x3e
    'NtGdiDdCreateSurfaceObject', # 0x3f
    'NtGdiDdDeleteDirectDrawObject', # 0x40
    'NtGdiDdDeleteSurfaceObject', # 0x41
    'NtGdiDdDestroyMoComp', # 0x42
    'NtGdiDdDestroySurface', # 0x43
    'NtGdiDdDestroyD3DBuffer', # 0x44
    'NtGdiDdEndMoCompFrame', # 0x45
    'NtGdiDdFlip', # 0x46
    'NtGdiDdFlipToGDISurface', # 0x47
    'NtGdiDdGetAvailDriverMemory', # 0x48
    'NtGdiDdGetBltStatus', # 0x49
    'NtGdiDdGetDC', # 0x4a
    'NtGdiDdGetDriverInfo', # 0x4b
    'NtGdiDdGetDxHandle', # 0x4c
    'NtGdiDdGetFlipStatus', # 0x4d
    'NtGdiDdGetInternalMoCompInfo', # 0x4e
    'NtGdiDdGetMoCompBuffInfo', # 0x4f
    'NtGdiDdGetMoCompGuids', # 0x50
    'NtGdiDdGetMoCompFormats', # 0x51
    'NtGdiDdGetScanLine', # 0x52
    'NtGdiDdLock', # 0x53
    'NtGdiDdLockD3D', # 0x54
    'NtGdiDdQueryDirectDrawObject', # 0x55
    'NtGdiDdQueryMoCompStatus', # 0x56
    'NtGdiDdReenableDirectDrawObject', # 0x57
    'NtGdiDdReleaseDC', # 0x58
    'NtGdiDdRenderMoComp', # 0x59
    'NtGdiDdResetVisrgn', # 0x5a
    'NtGdiDdSetColorKey', # 0x5b
    'NtGdiDdSetExclusiveMode', # 0x5c
    'NtGdiDdSetGammaRamp', # 0x5d
    'NtGdiDdCreateSurfaceEx', # 0x5e
    'NtGdiDdSetOverlayPosition', # 0x5f
    'NtGdiDdUnattachSurface', # 0x60
    'NtGdiDdUnlock', # 0x61
    'NtGdiDdUnlockD3D', # 0x62
    'NtGdiDdUpdateOverlay', # 0x63
    'NtGdiDdWaitForVerticalBlank', # 0x64
    'NtGdiDvpCanCreateVideoPort', # 0x65
    'NtGdiDvpColorControl', # 0x66
    'NtGdiDvpCreateVideoPort', # 0x67
    'NtGdiDvpDestroyVideoPort', # 0x68
    'NtGdiDvpFlipVideoPort', # 0x69
    'NtGdiDvpGetVideoPortBandwidth', # 0x6a
    'NtGdiDvpGetVideoPortField', # 0x6b
    'NtGdiDvpGetVideoPortFlipStatus', # 0x6c
    'NtGdiDvpGetVideoPortInputFormats', # 0x6d
    'NtGdiDvpGetVideoPortLine', # 0x6e
    'NtGdiDvpGetVideoPortOutputFormats', # 0x6f
    'NtGdiDvpGetVideoPortConnectInfo', # 0x70
    'NtGdiDvpGetVideoSignalStatus', # 0x71
    'NtGdiDvpUpdateVideoPort', # 0x72
    'NtGdiDvpWaitForVideoPortSync', # 0x73
    'NtGdiDvpAcquireNotification', # 0x74
    'NtGdiDvpReleaseNotification', # 0x75
    'NtGdiDxgGenericThunk', # 0x76
    'NtGdiDeleteClientObj', # 0x77
    'NtGdiDeleteColorSpace', # 0x78
    'NtGdiDeleteColorTransform', # 0x79
    'NtGdiDeleteObjectApp', # 0x7a
    'NtGdiDescribePixelFormat', # 0x7b
    'NtGdiGetPerBandInfo', # 0x7c
    'NtGdiDoBanding', # 0x7d
    'NtGdiDoPalette', # 0x7e
    'NtGdiDrawEscape', # 0x7f
    'NtGdiEllipse', # 0x80
    'NtGdiEnableEudc', # 0x81
    'NtGdiEndDoc', # 0x82
    'NtGdiEndPage', # 0x83
    'NtGdiEndPath', # 0x84
    'NtGdiEnumFontChunk', # 0x85
    'NtGdiEnumFontClose', # 0x86
    'NtGdiEnumFontOpen', # 0x87
    'NtGdiEnumObjects', # 0x88
    'NtGdiEqualRgn', # 0x89
    'NtGdiEudcLoadUnloadLink', # 0x8a
    'NtGdiExcludeClipRect', # 0x8b
    'NtGdiExtCreatePen', # 0x8c
    'NtGdiExtCreateRegion', # 0x8d
    'NtGdiExtEscape', # 0x8e
    'NtGdiExtFloodFill', # 0x8f
    'NtGdiExtGetObjectW', # 0x90
    'NtGdiExtSelectClipRgn', # 0x91
    'NtGdiExtTextOutW', # 0x92
    'NtGdiFillPath', # 0x93
    'NtGdiFillRgn', # 0x94
    'NtGdiFlattenPath', # 0x95
    'NtGdiFlushUserBatch', # 0x96
    'NtGdiFlush', # 0x97
    'NtGdiForceUFIMapping', # 0x98
    'NtGdiFrameRgn', # 0x99
    'NtGdiFullscreenControl', # 0x9a
    'NtGdiGetAndSetDCDword', # 0x9b
    'NtGdiGetAppClipBox', # 0x9c
    'NtGdiGetBitmapBits', # 0x9d
    'NtGdiGetBitmapDimension', # 0x9e
    'NtGdiGetBoundsRect', # 0x9f
    'NtGdiGetCharABCWidthsW', # 0xa0
    'NtGdiGetCharacterPlacementW', # 0xa1
    'NtGdiGetCharSet', # 0xa2
    'NtGdiGetCharWidthW', # 0xa3
    'NtGdiGetCharWidthInfo', # 0xa4
    'NtGdiGetColorAdjustment', # 0xa5
    'NtGdiGetColorSpaceforBitmap', # 0xa6
    'NtGdiGetDCDword', # 0xa7
    'NtGdiGetDCforBitmap', # 0xa8
    'NtGdiGetDCObject', # 0xa9
    'NtGdiGetDCPoint', # 0xaa
    'NtGdiGetDeviceCaps', # 0xab
    'NtGdiGetDeviceGammaRamp', # 0xac
    'NtGdiGetDeviceCapsAll', # 0xad
    'NtGdiGetDIBitsInternal', # 0xae
    'NtGdiGetETM', # 0xaf
    'NtGdiGetEudcTimeStampEx', # 0xb0
    'NtGdiGetFontData', # 0xb1
    'NtGdiGetFontResourceInfoInternalW', # 0xb2
    'NtGdiGetGlyphIndicesW', # 0xb3
    'NtGdiGetGlyphIndicesWInternal', # 0xb4
    'NtGdiGetGlyphOutline', # 0xb5
    'NtGdiGetKerningPairs', # 0xb6
    'NtGdiGetLinkedUFIs', # 0xb7
    'NtGdiGetMiterLimit', # 0xb8
    'NtGdiGetMonitorID', # 0xb9
    'NtGdiGetNearestColor', # 0xba
    'NtGdiGetNearestPaletteIndex', # 0xbb
    'NtGdiGetObjectBitmapHandle', # 0xbc
    'NtGdiGetOutlineTextMetricsInternalW', # 0xbd
    'NtGdiGetPath', # 0xbe
    'NtGdiGetPixel', # 0xbf
    'NtGdiGetRandomRgn', # 0xc0
    'NtGdiGetRasterizerCaps', # 0xc1
    'NtGdiGetRealizationInfo', # 0xc2
    'NtGdiGetRegionData', # 0xc3
    'NtGdiGetRgnBox', # 0xc4
    'NtGdiGetServerMetaFileBits', # 0xc5
    'NtGdiGetSpoolMessage', # 0xc6
    'NtGdiGetStats', # 0xc7
    'NtGdiGetStockObject', # 0xc8
    'NtGdiGetStringBitmapW', # 0xc9
    'NtGdiGetSystemPaletteUse', # 0xca
    'NtGdiGetTextCharsetInfo', # 0xcb
    'NtGdiGetTextExtent', # 0xcc
    'NtGdiGetTextExtentExW', # 0xcd
    'NtGdiGetTextFaceW', # 0xce
    'NtGdiGetTextMetricsW', # 0xcf
    'NtGdiGetTransform', # 0xd0
    'NtGdiGetUFI', # 0xd1
    'NtGdiGetEmbUFI', # 0xd2
    'NtGdiGetUFIPathname', # 0xd3
    'NtGdiGetEmbedFonts', # 0xd4
    'NtGdiChangeGhostFont', # 0xd5
    'NtGdiAddEmbFontToDC', # 0xd6
    'NtGdiGetFontUnicodeRanges', # 0xd7
    'NtGdiGetWidthTable', # 0xd8
    'NtGdiGradientFill', # 0xd9
    'NtGdiHfontCreate', # 0xda
    'NtGdiIcmBrushInfo', # 0xdb
    'NtGdiInit', # 0xdc
    'NtGdiInitSpool', # 0xdd
    'NtGdiIntersectClipRect', # 0xde
    'NtGdiInvertRgn', # 0xdf
    'NtGdiLineTo', # 0xe0
    'NtGdiMakeFontDir', # 0xe1
    'NtGdiMakeInfoDC', # 0xe2
    'NtGdiMaskBlt', # 0xe3
    'NtGdiModifyWorldTransform', # 0xe4
    'NtGdiMonoBitmap', # 0xe5
    'NtGdiMoveTo', # 0xe6
    'NtGdiOffsetClipRgn', # 0xe7
    'NtGdiOffsetRgn', # 0xe8
    'NtGdiOpenDCW', # 0xe9
    'NtGdiPatBlt', # 0xea
    'NtGdiPolyPatBlt', # 0xeb
    'NtGdiPathToRegion', # 0xec
    'NtGdiPlgBlt', # 0xed
    'NtGdiPolyDraw', # 0xee
    'NtGdiPolyPolyDraw', # 0xef
    'NtGdiPolyTextOutW', # 0xf0
    'NtGdiPtInRegion', # 0xf1
    'NtGdiPtVisible', # 0xf2
    'NtGdiQueryFonts', # 0xf3
    'NtGdiQueryFontAssocInfo', # 0xf4
    'NtGdiRectangle', # 0xf5
    'NtGdiRectInRegion', # 0xf6
    'NtGdiRectVisible', # 0xf7
    'NtGdiRemoveFontResourceW', # 0xf8
    'NtGdiRemoveFontMemResourceEx', # 0xf9
    'NtGdiResetDC', # 0xfa
    'NtGdiResizePalette', # 0xfb
    'NtGdiRestoreDC', # 0xfc
    'NtGdiRoundRect', # 0xfd
    'NtGdiSaveDC', # 0xfe
    'NtGdiScaleViewportExtEx', # 0xff
    'NtGdiScaleWindowExtEx', # 0x100
    'NtGdiSelectBitmap', # 0x101
    'NtGdiSelectBrush', # 0x102
    'NtGdiSelectClipPath', # 0x103
    'NtGdiSelectFont', # 0x104
    'NtGdiSelectPen', # 0x105
    'NtGdiSetBitmapAttributes', # 0x106
    'NtGdiSetBitmapBits', # 0x107
    'NtGdiSetBitmapDimension', # 0x108
    'NtGdiSetBoundsRect', # 0x109
    'NtGdiSetBrushAttributes', # 0x10a
    'NtGdiSetBrushOrg', # 0x10b
    'NtGdiSetColorAdjustment', # 0x10c
    'NtGdiSetColorSpace', # 0x10d
    'NtGdiSetDeviceGammaRamp', # 0x10e
    'NtGdiSetDIBitsToDeviceInternal', # 0x10f
    'NtGdiSetFontEnumeration', # 0x110
    'NtGdiSetFontXform', # 0x111
    'NtGdiSetIcmMode', # 0x112
    'NtGdiSetLinkedUFIs', # 0x113
    'NtGdiSetMagicColors', # 0x114
    'NtGdiSetMetaRgn', # 0x115
    'NtGdiSetMiterLimit', # 0x116
    'NtGdiGetDeviceWidth', # 0x117
    'NtGdiMirrorWindowOrg', # 0x118
    'NtGdiSetLayout', # 0x119
    'NtGdiSetPixel', # 0x11a
    'NtGdiSetPixelFormat', # 0x11b
    'NtGdiSetRectRgn', # 0x11c
    'NtGdiSetSystemPaletteUse', # 0x11d
    'NtGdiSetTextJustification', # 0x11e
    'NtGdiSetupPublicCFONT', # 0x11f
    'NtGdiSetVirtualResolution', # 0x120
    'NtGdiSetSizeDevice', # 0x121
    'NtGdiStartDoc', # 0x122
    'NtGdiStartPage', # 0x123
    'NtGdiStretchBlt', # 0x124
    'NtGdiStretchDIBitsInternal', # 0x125
    'NtGdiStrokeAndFillPath', # 0x126
    'NtGdiStrokePath', # 0x127
    'NtGdiSwapBuffers', # 0x128
    'NtGdiTransformPoints', # 0x129
    'NtGdiTransparentBlt', # 0x12a
    'NtGdiUnloadPrinterDriver', # 0x12b
    'NtGdiUnmapMemFont', # 0x12c
    'NtGdiUnrealizeObject', # 0x12d
    'NtGdiUpdateColors', # 0x12e
    'NtGdiWidenPath', # 0x12f
    'NtUserActivateKeyboardLayout', # 0x130
    'NtUserAlterWindowStyle', # 0x131
    'NtUserAssociateInputContext', # 0x132
    'NtUserAttachThreadInput', # 0x133
    'NtUserBeginPaint', # 0x134
    'NtUserBitBltSysBmp', # 0x135
    'NtUserBlockInput', # 0x136
    'NtUserBuildHimcList', # 0x137
    'NtUserBuildHwndList', # 0x138
    'NtUserBuildNameList', # 0x139
    'NtUserBuildPropList', # 0x13a
    'NtUserCallHwnd', # 0x13b
    'NtUserCallHwndLock', # 0x13c
    'NtUserCallHwndOpt', # 0x13d
    'NtUserCallHwndParam', # 0x13e
    'NtUserCallHwndParamLock', # 0x13f
    'NtUserCallMsgFilter', # 0x140
    'NtUserCallNextHookEx', # 0x141
    'NtUserCallNoParam', # 0x142
    'NtUserCallOneParam', # 0x143
    'NtUserCallTwoParam', # 0x144
    'NtUserChangeClipboardChain', # 0x145
    'NtUserChangeDisplaySettings', # 0x146
    'NtUserCheckImeHotKey', # 0x147
    'NtUserCheckMenuItem', # 0x148
    'NtUserChildWindowFromPointEx', # 0x149
    'NtUserClipCursor', # 0x14a
    'NtUserCloseClipboard', # 0x14b
    'NtUserCloseDesktop', # 0x14c
    'NtUserCloseWindowStation', # 0x14d
    'NtUserConsoleControl', # 0x14e
    'NtUserConvertMemHandle', # 0x14f
    'NtUserCopyAcceleratorTable', # 0x150
    'NtUserCountClipboardFormats', # 0x151
    'NtUserCreateAcceleratorTable', # 0x152
    'NtUserCreateCaret', # 0x153
    'NtUserCreateDesktop', # 0x154
    'NtUserCreateInputContext', # 0x155
    'NtUserCreateLocalMemHandle', # 0x156
    'NtUserCreateWindowEx', # 0x157
    'NtUserCreateWindowStation', # 0x158
    'NtUserDdeGetQualityOfService', # 0x159
    'NtUserDdeInitialize', # 0x15a
    'NtUserDdeSetQualityOfService', # 0x15b
    'NtUserDeferWindowPos', # 0x15c
    'NtUserDefSetText', # 0x15d
    'NtUserDeleteMenu', # 0x15e
    'NtUserDestroyAcceleratorTable', # 0x15f
    'NtUserDestroyCursor', # 0x160
    'NtUserDestroyInputContext', # 0x161
    'NtUserDestroyMenu', # 0x162
    'NtUserDestroyWindow', # 0x163
    'NtUserDisableThreadIme', # 0x164
    'NtUserDispatchMessage', # 0x165
    'NtUserDragDetect', # 0x166
    'NtUserDragObject', # 0x167
    'NtUserDrawAnimatedRects', # 0x168
    'NtUserDrawCaption', # 0x169
    'NtUserDrawCaptionTemp', # 0x16a
    'NtUserDrawIconEx', # 0x16b
    'NtUserDrawMenuBarTemp', # 0x16c
    'NtUserEmptyClipboard', # 0x16d
    'NtUserEnableMenuItem', # 0x16e
    'NtUserEnableScrollBar', # 0x16f
    'NtUserEndDeferWindowPosEx', # 0x170
    'NtUserEndMenu', # 0x171
    'NtUserEndPaint', # 0x172
    'NtUserEnumDisplayDevices', # 0x173
    'NtUserEnumDisplayMonitors', # 0x174
    'NtUserEnumDisplaySettings', # 0x175
    'NtUserEvent', # 0x176
    'NtUserExcludeUpdateRgn', # 0x177
    'NtUserFillWindow', # 0x178
    'NtUserFindExistingCursorIcon', # 0x179
    'NtUserFindWindowEx', # 0x17a
    'NtUserFlashWindowEx', # 0x17b
    'NtUserGetAltTabInfo', # 0x17c
    'NtUserGetAncestor', # 0x17d
    'NtUserGetAppImeLevel', # 0x17e
    'NtUserGetAsyncKeyState', # 0x17f
    'NtUserGetAtomName', # 0x180
    'NtUserGetCaretBlinkTime', # 0x181
    'NtUserGetCaretPos', # 0x182
    'NtUserGetClassInfo', # 0x183
    'NtUserGetClassName', # 0x184
    'NtUserGetClipboardData', # 0x185
    'NtUserGetClipboardFormatName', # 0x186
    'NtUserGetClipboardOwner', # 0x187
    'NtUserGetClipboardSequenceNumber', # 0x188
    'NtUserGetClipboardViewer', # 0x189
    'NtUserGetClipCursor', # 0x18a
    'NtUserGetComboBoxInfo', # 0x18b
    'NtUserGetControlBrush', # 0x18c
    'NtUserGetControlColor', # 0x18d
    'NtUserGetCPD', # 0x18e
    'NtUserGetCursorFrameInfo', # 0x18f
    'NtUserGetCursorInfo', # 0x190
    'NtUserGetDC', # 0x191
    'NtUserGetDCEx', # 0x192
    'NtUserGetDoubleClickTime', # 0x193
    'NtUserGetForegroundWindow', # 0x194
    'NtUserGetGuiResources', # 0x195
    'NtUserGetGUIThreadInfo', # 0x196
    'NtUserGetIconInfo', # 0x197
    'NtUserGetIconSize', # 0x198
    'NtUserGetImeHotKey', # 0x199
    'NtUserGetImeInfoEx', # 0x19a
    'NtUserGetInternalWindowPos', # 0x19b
    'NtUserGetKeyboardLayoutList', # 0x19c
    'NtUserGetKeyboardLayoutName', # 0x19d
    'NtUserGetKeyboardState', # 0x19e
    'NtUserGetKeyNameText', # 0x19f
    'NtUserGetKeyState', # 0x1a0
    'NtUserGetListBoxInfo', # 0x1a1
    'NtUserGetMenuBarInfo', # 0x1a2
    'NtUserGetMenuIndex', # 0x1a3
    'NtUserGetMenuItemRect', # 0x1a4
    'NtUserGetMessage', # 0x1a5
    'NtUserGetMouseMovePointsEx', # 0x1a6
    'NtUserGetObjectInformation', # 0x1a7
    'NtUserGetOpenClipboardWindow', # 0x1a8
    'NtUserGetPriorityClipboardFormat', # 0x1a9
    'NtUserGetProcessWindowStation', # 0x1aa
    'NtUserGetRawInputBuffer', # 0x1ab
    'NtUserGetRawInputData', # 0x1ac
    'NtUserGetRawInputDeviceInfo', # 0x1ad
    'NtUserGetRawInputDeviceList', # 0x1ae
    'NtUserGetRegisteredRawInputDevices', # 0x1af
    'NtUserGetScrollBarInfo', # 0x1b0
    'NtUserGetSystemMenu', # 0x1b1
    'NtUserGetThreadDesktop', # 0x1b2
    'NtUserGetThreadState', # 0x1b3
    'NtUserGetTitleBarInfo', # 0x1b4
    'NtUserGetUpdateRect', # 0x1b5
    'NtUserGetUpdateRgn', # 0x1b6
    'NtUserGetWindowDC', # 0x1b7
    'NtUserGetWindowPlacement', # 0x1b8
    'NtUserGetWOWClass', # 0x1b9
    'NtUserHardErrorControl', # 0x1ba
    'NtUserHideCaret', # 0x1bb
    'NtUserHiliteMenuItem', # 0x1bc
    'NtUserImpersonateDdeClientWindow', # 0x1bd
    'NtUserInitialize', # 0x1be
    'NtUserInitializeClientPfnArrays', # 0x1bf
    'NtUserInitTask', # 0x1c0
    'NtUserInternalGetWindowText', # 0x1c1
    'NtUserInvalidateRect', # 0x1c2
    'NtUserInvalidateRgn', # 0x1c3
    'NtUserIsClipboardFormatAvailable', # 0x1c4
    'NtUserKillTimer', # 0x1c5
    'NtUserLoadKeyboardLayoutEx', # 0x1c6
    'NtUserLockWindowStation', # 0x1c7
    'NtUserLockWindowUpdate', # 0x1c8
    'NtUserLockWorkStation', # 0x1c9
    'NtUserMapVirtualKeyEx', # 0x1ca
    'NtUserMenuItemFromPoint', # 0x1cb
    'NtUserMessageCall', # 0x1cc
    'NtUserMinMaximize', # 0x1cd
    'NtUserMNDragLeave', # 0x1ce
    'NtUserMNDragOver', # 0x1cf
    'NtUserModifyUserStartupInfoFlags', # 0x1d0
    'NtUserMoveWindow', # 0x1d1
    'NtUserNotifyIMEStatus', # 0x1d2
    'NtUserNotifyProcessCreate', # 0x1d3
    'NtUserNotifyWinEvent', # 0x1d4
    'NtUserOpenClipboard', # 0x1d5
    'NtUserOpenDesktop', # 0x1d6
    'NtUserOpenInputDesktop', # 0x1d7
    'NtUserOpenWindowStation', # 0x1d8
    'NtUserPaintDesktop', # 0x1d9
    'NtUserPeekMessage', # 0x1da
    'NtUserPostMessage', # 0x1db
    'NtUserPostThreadMessage', # 0x1dc
    'NtUserPrintWindow', # 0x1dd
    'NtUserProcessConnect', # 0x1de
    'NtUserQueryInformationThread', # 0x1df
    'NtUserQueryInputContext', # 0x1e0
    'NtUserQuerySendMessage', # 0x1e1
    'NtUserQueryUserCounters', # 0x1e2
    'NtUserQueryWindow', # 0x1e3
    'NtUserRealChildWindowFromPoint', # 0x1e4
    'NtUserRealInternalGetMessage', # 0x1e5
    'NtUserRealWaitMessageEx', # 0x1e6
    'NtUserRedrawWindow', # 0x1e7
    'NtUserRegisterClassExWOW', # 0x1e8
    'NtUserRegisterUserApiHook', # 0x1e9
    'NtUserRegisterHotKey', # 0x1ea
    'NtUserRegisterRawInputDevices', # 0x1eb
    'NtUserRegisterTasklist', # 0x1ec
    'NtUserRegisterWindowMessage', # 0x1ed
    'NtUserRemoveMenu', # 0x1ee
    'NtUserRemoveProp', # 0x1ef
    'NtUserResolveDesktop', # 0x1f0
    'NtUserResolveDesktopForWOW', # 0x1f1
    'NtUserSBGetParms', # 0x1f2
    'NtUserScrollDC', # 0x1f3
    'NtUserScrollWindowEx', # 0x1f4
    'NtUserSelectPalette', # 0x1f5
    'NtUserSendInput', # 0x1f6
    'NtUserSetActiveWindow', # 0x1f7
    'NtUserSetAppImeLevel', # 0x1f8
    'NtUserSetCapture', # 0x1f9
    'NtUserSetClassLong', # 0x1fa
    'NtUserSetClassWord', # 0x1fb
    'NtUserSetClipboardData', # 0x1fc
    'NtUserSetClipboardViewer', # 0x1fd
    'NtUserSetConsoleReserveKeys', # 0x1fe
    'NtUserSetCursor', # 0x1ff
    'NtUserSetCursorContents', # 0x200
    'NtUserSetCursorIconData', # 0x201
    'NtUserSetDbgTag', # 0x202
    'NtUserSetFocus', # 0x203
    'NtUserSetImeHotKey', # 0x204
    'NtUserSetImeInfoEx', # 0x205
    'NtUserSetImeOwnerWindow', # 0x206
    'NtUserSetInformationProcess', # 0x207
    'NtUserSetInformationThread', # 0x208
    'NtUserSetInternalWindowPos', # 0x209
    'NtUserSetKeyboardState', # 0x20a
    'NtUserSetLogonNotifyWindow', # 0x20b
    'NtUserSetMenu', # 0x20c
    'NtUserSetMenuContextHelpId', # 0x20d
    'NtUserSetMenuDefaultItem', # 0x20e
    'NtUserSetMenuFlagRtoL', # 0x20f
    'NtUserSetObjectInformation', # 0x210
    'NtUserSetParent', # 0x211
    'NtUserSetProcessWindowStation', # 0x212
    'NtUserSetProp', # 0x213
    'NtUserSetRipFlags', # 0x214
    'NtUserSetScrollInfo', # 0x215
    'NtUserSetShellWindowEx', # 0x216
    'NtUserSetSysColors', # 0x217
    'NtUserSetSystemCursor', # 0x218
    'NtUserSetSystemMenu', # 0x219
    'NtUserSetSystemTimer', # 0x21a
    'NtUserSetThreadDesktop', # 0x21b
    'NtUserSetThreadLayoutHandles', # 0x21c
    'NtUserSetThreadState', # 0x21d
    'NtUserSetTimer', # 0x21e
    'NtUserSetWindowFNID', # 0x21f
    'NtUserSetWindowLong', # 0x220
    'NtUserSetWindowPlacement', # 0x221
    'NtUserSetWindowPos', # 0x222
    'NtUserSetWindowRgn', # 0x223
    'NtUserSetWindowsHookAW', # 0x224
    'NtUserSetWindowsHookEx', # 0x225
    'NtUserSetWindowStationUser', # 0x226
    'NtUserSetWindowWord', # 0x227
    'NtUserSetWinEventHook', # 0x228
    'NtUserShowCaret', # 0x229
    'NtUserShowScrollBar', # 0x22a
    'NtUserShowWindow', # 0x22b
    'NtUserShowWindowAsync', # 0x22c
    'NtUserSoundSentry', # 0x22d
    'NtUserSwitchDesktop', # 0x22e
    'NtUserSystemParametersInfo', # 0x22f
    'NtUserTestForInteractiveUser', # 0x230
    'NtUserThunkedMenuInfo', # 0x231
    'NtUserThunkedMenuItemInfo', # 0x232
    'NtUserToUnicodeEx', # 0x233
    'NtUserTrackMouseEvent', # 0x234
    'NtUserTrackPopupMenuEx', # 0x235
    'NtUserCalcMenuBar', # 0x236
    'NtUserPaintMenuBar', # 0x237
    'NtUserTranslateAccelerator', # 0x238
    'NtUserTranslateMessage', # 0x239
    'NtUserUnhookWindowsHookEx', # 0x23a
    'NtUserUnhookWinEvent', # 0x23b
    'NtUserUnloadKeyboardLayout', # 0x23c
    'NtUserUnlockWindowStation', # 0x23d
    'NtUserUnregisterClass', # 0x23e
    'NtUserUnregisterUserApiHook', # 0x23f
    'NtUserUnregisterHotKey', # 0x240
    'NtUserUpdateInputContext', # 0x241
    'NtUserUpdateInstance', # 0x242
    'NtUserUpdateLayeredWindow', # 0x243
    'NtUserGetLayeredWindowAttributes', # 0x244
    'NtUserSetLayeredWindowAttributes', # 0x245
    'NtUserUpdatePerUserSystemParameters', # 0x246
    'NtUserUserHandleGrantAccess', # 0x247
    'NtUserValidateHandleSecure', # 0x248
    'NtUserValidateRect', # 0x249
    'NtUserValidateTimerCallback', # 0x24a
    'NtUserVkKeyScanEx', # 0x24b
    'NtUserWaitForInputIdle', # 0x24c
    'NtUserWaitForMsgAndEvent', # 0x24d
    'NtUserWaitMessage', # 0x24e
    'NtUserWin32PoolAllocationStats', # 0x24f
    'NtUserWindowFromPoint', # 0x250
    'NtUserYieldTask', # 0x251
    'NtUserRemoteConnect', # 0x252
    'NtUserRemoteRedrawRectangle', # 0x253
    'NtUserRemoteRedrawScreen', # 0x254
    'NtUserRemoteStopScreenUpdates', # 0x255
    'NtUserCtxDisplayIOCtl', # 0x256
    'NtGdiEngAssociateSurface', # 0x257
    'NtGdiEngCreateBitmap', # 0x258
    'NtGdiEngCreateDeviceSurface', # 0x259
    'NtGdiEngCreateDeviceBitmap', # 0x25a
    'NtGdiEngCreatePalette', # 0x25b
    'NtGdiEngComputeGlyphSet', # 0x25c
    'NtGdiEngCopyBits', # 0x25d
    'NtGdiEngDeletePalette', # 0x25e
    'NtGdiEngDeleteSurface', # 0x25f
    'NtGdiEngEraseSurface', # 0x260
    'NtGdiEngUnlockSurface', # 0x261
    'NtGdiEngLockSurface', # 0x262
    'NtGdiEngBitBlt', # 0x263
    'NtGdiEngStretchBlt', # 0x264
    'NtGdiEngPlgBlt', # 0x265
    'NtGdiEngMarkBandingSurface', # 0x266
    'NtGdiEngStrokePath', # 0x267
    'NtGdiEngFillPath', # 0x268
    'NtGdiEngStrokeAndFillPath', # 0x269
    'NtGdiEngPaint', # 0x26a
    'NtGdiEngLineTo', # 0x26b
    'NtGdiEngAlphaBlend', # 0x26c
    'NtGdiEngGradientFill', # 0x26d
    'NtGdiEngTransparentBlt', # 0x26e
    'NtGdiEngTextOut', # 0x26f
    'NtGdiEngStretchBltROP', # 0x270
    'NtGdiXLATEOBJ_cGetPalette', # 0x271
    'NtGdiXLATEOBJ_iXlate', # 0x272
    'NtGdiXLATEOBJ_hGetColorTransform', # 0x273
    'NtGdiCLIPOBJ_bEnum', # 0x274
    'NtGdiCLIPOBJ_cEnumStart', # 0x275
    'NtGdiCLIPOBJ_ppoGetPath', # 0x276
    'NtGdiEngDeletePath', # 0x277
    'NtGdiEngCreateClip', # 0x278
    'NtGdiEngDeleteClip', # 0x279
    'NtGdiBRUSHOBJ_ulGetBrushColor', # 0x27a
    'NtGdiBRUSHOBJ_pvAllocRbrush', # 0x27b
    'NtGdiBRUSHOBJ_pvGetRbrush', # 0x27c
    'NtGdiBRUSHOBJ_hGetColorTransform', # 0x27d
    'NtGdiXFORMOBJ_bApplyXform', # 0x27e
    'NtGdiXFORMOBJ_iGetXform', # 0x27f
    'NtGdiFONTOBJ_vGetInfo', # 0x280
    'NtGdiFONTOBJ_pxoGetXform', # 0x281
    'NtGdiFONTOBJ_cGetGlyphs', # 0x282
    'NtGdiFONTOBJ_pifi', # 0x283
    'NtGdiFONTOBJ_pfdg', # 0x284
    'NtGdiFONTOBJ_pQueryGlyphAttrs', # 0x285
    'NtGdiFONTOBJ_pvTrueTypeFontFile', # 0x286
    'NtGdiFONTOBJ_cGetAllGlyphHandles', # 0x287
    'NtGdiSTROBJ_bEnum', # 0x288
    'NtGdiSTROBJ_bEnumPositionsOnly', # 0x289
    'NtGdiSTROBJ_bGetAdvanceWidths', # 0x28a
    'NtGdiSTROBJ_vEnumStart', # 0x28b
    'NtGdiSTROBJ_dwGetCodePage', # 0x28c
    'NtGdiPATHOBJ_vGetBounds', # 0x28d
    'NtGdiPATHOBJ_bEnum', # 0x28e
    'NtGdiPATHOBJ_vEnumStart', # 0x28f
    'NtGdiPATHOBJ_vEnumStartClipLines', # 0x290
    'NtGdiPATHOBJ_bEnumClipLines', # 0x291
    'NtGdiGetDhpdev', # 0x292
    'NtGdiEngCheckAbort', # 0x293
    'NtGdiHT_Get8BPPFormatPalette', # 0x294
    'NtGdiHT_Get8BPPMaskPalette', # 0x295
    'NtGdiUpdateTransform', # 0x296
    'NtGdiSetPUMPDOBJ', # 0x297
    'NtGdiBRUSHOBJ_DeleteRbrush', # 0x298
    'NtGdiUMPDEngFreeUserMem', # 0x299
    'NtGdiDrawStream', # 0x29a
    ],
]



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""

import socket
import volatility.obj as obj
import volatility.plugins.linux.common as linux_common
import volatility.plugins.linux.lsof as linux_lsof
import volatility.plugins.linux.pslist as linux_pslist

class linux_netstat(linux_pslist.linux_pslist):
    """Lists open sockets"""
    
    def __init__(self, config, *args, **kwargs):
        linux_pslist.linux_pslist.__init__(self, config, *args, **kwargs)
        self._config.add_option('IGNORE_UNIX', short_option = 'U', default = None, help = 'ignore unix sockets', action = 'store_true')
    
    def calculate(self):
        linux_common.set_plugin_members(self)
        if not self.profile.has_type("inet_sock"):
            # ancient (2.6.9) centos kernels do not have inet_sock in debug info
            raise AttributeError, "Given profile does not have inet_sock, please file a bug if the kernel version is > 2.6.11"

        openfiles = linux_lsof.linux_lsof(self._config).calculate()

        for (task, filp, i) in openfiles:

            # its a socket!
            if filp.f_op == self.addr_space.profile.get_symbol("socket_file_ops") or filp.dentry.d_op == self.addr_space.profile.get_symbol("sockfs_dentry_operations"):

                iaddr = filp.dentry.d_inode
                skt = self.SOCKET_I(iaddr)
                inet_sock = obj.Object("inet_sock", offset = skt.sk, vm = self.addr_space)

                yield task, i, inet_sock

    def render_text(self, outfd, data):

        for task, _fd, inet_sock in data:

            if inet_sock.protocol in ("TCP", "UDP", "IP", "HOPOPT"): #hopopt is where unix sockets end up on linux

                state = inet_sock.state if inet_sock.protocol == "TCP" else ""
                family = inet_sock.sk.__sk_common.skc_family #pylint: disable-msg=W0212

                if family == socket.AF_UNIX:

                    # the user choose to ignore unix sockets
                    if self._config.IGNORE_UNIX:
                        continue

                    unix_sock = obj.Object("unix_sock", offset = inet_sock.sk.v(), vm = self.addr_space)

                    if unix_sock.addr:

                        name = obj.Object("sockaddr_un", offset = unix_sock.addr.name.obj_offset, vm = self.addr_space)

                        # only print out sockets with paths
                        if str(name.sun_path) != "":
                            outfd.write("UNIX {0:s}\n".format(name.sun_path))

                elif family in (socket.AF_INET, socket.AF_INET6):

                    sport = inet_sock.src_port 
                    dport = inet_sock.dst_port 
                    saddr = inet_sock.src_addr
                    daddr = inet_sock.dst_addr

                    outfd.write("{0:8s} {1}:{2:<5} {3}:{4:<5} {5:s} {6:>17s}/{7:<5d}\n".format(inet_sock.protocol, saddr, sport, daddr, dport, state, task.comm, task.pid))

                #else:
                #    print "unknown family: %d" % family

    # has to get the struct socket given an inode (see SOCKET_I in sock.h)
    def SOCKET_I(self, inode):
        # if too many of these, write a container_of
        backsize = self.profile.get_obj_size("socket")
        addr = inode - backsize

        return obj.Object('socket', offset = addr, vm = self.addr_space)



# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.

""" This module implements the volatility caching subsystem.

The volatility caching subsystem has the following design goals:

 1) Ability to cache arbitrary objects - The allows complex objects to
    be cached for later retrieval. For example, objects may be as
    simple as constants for KPCR addresses, to entire x86 page
    translation tables, or even hibernation decompression
    datastructures. To achieve this we use the standard python pickle
    system. In many use cases, the cache needs to facilitate
    persistant memoising of functions and generators (more on that
    below).

 2) Cached objects are stored by a hierarchical key namespace. Keys
    are specified in a URL notation. By default, relative URLs are
    interpreted relative to the memory image location (the value of
    the --location option). This scheme allows us to specify both
    global (per installation) and per image keys. For example given an
    image located in /tmp/foobar.img:

    - file:///tmp/foobar.img/kernel/debugging/KPCR refers to this
      image's KPCR location.

    - file:///tmp/foobar.img/address_spaces/memory_translation/pdpte
      refers to the cached page tables.

    - http://www.volatility.org/schema#configuration/renderer specifies
      the currently configured renderer (i.e. its a global setting).

 3) Storage of the cache is abstracted and selectable via the
 --cache_engine configuration variable. This allows the separation
 from the concerete storage of the cache and the abstraction of the
 cache in a running process.

Abstraction of Cache
--------------------

Within the running volatiltiy framework the cache appears as an
abstract tree with nodes inherited from the CacheNode class:

class CacheNode(object):
    def __init__(self, name, parent, payload = None):
        ''' Creates a new Cache node under the parent. The new node
        will carry the specified payload
        '''

    def __str__(self):
        ''' Produce a human readable version of the payload '''

    def set_payload(self, payload):
        ''' Update the current payload with the new specified payload '''

    def dump(self):
        ''' Dump the node to disk for later retrieval. This is
        normally called when the process has exited. '''

    def get_payload(self):
       ''' retrieve this node's payload '''

In order to check the cache, plugins issue the Cache.Check() function:

def Check(path, callback = None, cls = CacheNode):
    ''' Traverse the cache tree and retrieve the stored CacheNode.

    If there is no such stored CacheNode and callback is specified,
    attempt to create it using the cache_node_class with the payload
    returned from the callback. If callback is not specified we just
    return None.

Decorators
----------
You can also use the cache decorator to cache the results of any
function - this is probably the easiest way to apply caching to
existing code. For example, suppose we want to cache the results of
the psscan plugin:

class PSScan(commands.Command):
....
   @cache("/scanners/psscan")
   def calculate(self):
       .....

This will automatically create the CacheNode at the specified tree
location (note that since the URL is given as a relative URL it is
based at the current value of the --location - that means it applies
to the current memory image only).

Note that since calculate() returns a generator, the decorator will
also return a generator - It will not iterate over the calculate
method unnecessarily, but will yield results immediately. This does
not compromise performance in the case of a cache miss. Unfortunately
this also means that if the generator is stopped prematurely, we are
unable to cache the result set in the general case. This is the only
caveat on caching generators.

Storage classes
---------------
The cache system discussed above can be thought of as an abstract
construct in the process memory. To make it persistant on disk we have
the storage class (which can be selected using the --cache_engine
directive). The following cache engines are implemented:

File Storage
============
This is the default cache engine. We simply maintain a directory
structure which corresponds to the URL of the key after applying the
appropriate filesystem safe escaping operation. Objects are stored in
stand alone files using the pickle module.

Zip Storage
===========
This storage is essentially the same as the File storage above, except
that the cache directory for each image file is maintained in a Zip
file stored at the --cache_direcory directive with the same filename
as the image and a .zip extension.


Use cases
---------
The following common use cases are discussed:

1) Dynamic address spaces. In some address spaces memory address
   mappings can not be cached since they change all the time. For
   example in the firewire address space, it is incorrect to cache any
   page translations or scanning results etc. This is easily achieved
   by having the firewire address space store a BlockingCacheNode()
   instance at critical tree nodes. These prevent new nodes from being
   inserted into the tree and force a cache miss whenever any keys are
   searched under these nodes. Note that this still allows the cache
   to store the locations of things which might not change, even for
   live memory analysis, such as KPCR locations.

2) History logging and audit logs. Currently volatility works by
   running the framework multiple times on the same plugin with
   different command line options. This can be audited using the
   caching system by storing the current command line in a specific
   location using a specific CacheNode. This implementation can be
   used to append new commandlines to the same key. Configuration
   options can also become sticky in this way and remember the same
   values they had previously. This avoid users having to append many
   command line arguements (i.e. having to specify --profile, --kpcr,
   --dtb on every command line).

3) Unit tests.  Unit tests can be easily implemented using the caching
   subsystem as follows:

   - A test() method is added to each plugin. Usually this is actually
     the same as calculate().

   - This method is decorated to be cached under the
     "/tests/pluginname" key (i.e. relative to the current image). The
     CacheNode implementation is TestCacheNode which implements a
     special update_payload() method. The TestCacheNode also ensures
     that cache miss always occurs (by implementing a get_payload()
     method which returns None).

   - The update_payload() method ensures that the old payload and the
     new payloads are the same (if they are generators we ensure each
     member is the same as well - using the __eq__ method).

   The overall result is that unit tests can be run on any image as
   normal. If the particular test was never run on the image, we just
   cache the result of the plugin. If on the other hand, the result
   was already run on this image, the old result is compared to the
   new result and if a discrepancy is detected, an exception is
   raised.

   This testing framework is easy to implement and automatically
   guards against regression bugs. Since we use the __eq__ method of
   arbitrary objects, its also not limited to testing text string
   matches. For example, the object framework defines two objects are
   being equal if they are of the same type and they point at the same
   address. Even if the textual representation of the object's
   printouts has changed between versions, as long as the same objects
   are found in both cases no regressions will be reported.

4) Reporting framework. By having a persistant caching framework we
   now have the concept of a volatility analysis session. In other
   words, each new execution of volatility adds new information to
   what we know about the image. This new information is stored in the
   cache tree. We can actually produce a full report from the cache
   tree by traversing all the CacheNodes and calling their __str__()
   methods.

   If caching is introduced via decorators, the CacheNode already
   knows about the render() method of the plugin and can automatically
   generate the output from the plugin (this is very fast as the
   calculate is received from the cache). We therefore can generate a
   full report of all the plugins very quickly automatically.

   By default CacheNodes have an empty __str__() methods, so things
   like pas2kas lookup tables are not reported. Specialised reporting
   functions can be made if needed by implementing __str__() functions
   as needed.

"""
import types
import os
import urlparse
import volatility.conf as conf
import volatility.obj as obj
import volatility.debug as debug
import volatility.exceptions as exceptions
import cPickle as pickle
config = conf.ConfObject()

## Where to stick the cache
default_cache_location = os.path.join((os.environ.get("XDG_CACHE_HOME") or os.path.expanduser("~/.cache")), "volatility")

config.add_option("CACHE-DIRECTORY", default = default_cache_location,
                  cache_invalidator = False,
                  help = "Directory where cache files are stored")

class CacheContainsGenerator(exceptions.VolatilityException):
    """Exception raised when the cache contains a generator"""
    pass

class InvalidCache(Exception):
    """Exception raised when the cache item is determined to be invalid."""
    pass

class CacheNode(object):
    """ Base class for Cache nodes """
    def __init__(self, name, stem, storage = None, payload = None, invalidator = None):
        ''' Creates a new Cache node under the parent. The new node
        will carry the specified payload
        '''
        self.name = name
        self.payload = payload
        self.storage = storage
        self.stem = stem

        # This object encapsulate the running environment. If the
        # environment during the time of unpickling differs from the
        # environment during the time of pickling we refuse to
        # unpickle this object, and the cache misses. We dont really
        # do anything with it, just have it serialised as well.
        self.invalidator = invalidator

    def __getitem__(self, item = ''):
        item_url = "{0}/{1}".format(self.stem, item)

        ## Try to load it from the storage manager
        try:
            result = self.storage.load(item_url)
            if result:
                return result
        except Exception, e:
            raise KeyError(e)

        ## Make a new empty Node instead on demand
        raise KeyError("item not found")

    def __str__(self):
        ''' Produce a human readable version of the payload. '''
        return ''

    def _find_generators(self, item):
        """ A recursive function to flatten generators into lists """
        try:
            result = []
            # Make sure dicts aren't flattened to lists
            if isinstance(item, dict):
                result = {}
                for i in item:
                    result[self._find_generators(i)] = self._find_generators(item[i])
                return result

            # Since NoneObjects and strings are both iterable, treat them specially
            if isinstance(item, obj.NoneObject) or isinstance(item, str):
                return item

            if isinstance(item, types.GeneratorType):
                raise CacheContainsGenerator
            for x in iter(item):
                flat_x = self._find_generators(x)
                result.append(flat_x)

            return result
        except TypeError:
            return item

    def set_payload(self, payload):
        ''' Update the current payload with the new specified payload '''
        try:
            self.payload = self._find_generators(payload)
        except CacheContainsGenerator:
            # This only works because None payload cached results are rerun
            self.payload = None

    def dump(self):
        ''' Dump the node to disk for later retrieval. This is
        normally called when the process has exited. '''
        if self.payload:
            self.storage.dump(self.stem, self)

    def get_payload(self):
        """Retrieve this node's payload"""
        return self.payload

class BlockingNode(CacheNode):
    """Node that fails on all cache attempts and no-ops on cache storage attempts"""
    def __init__(self, name, stem, **kwargs):
        CacheNode.__init__(self, name, stem, **kwargs)

    def __getitem__(self, item = ''):
        return BlockingNode(item, '/'.join((self.stem, item)))

    def dump(self):
        """Ensure nothing gets dumped"""
        pass

    def get_payload(self):
        """Do not set a payload for a blocked cache node"""
        pass

class Invalidator(object):
    """ The Invalidator encapsulates program state to control
    invalidation of the cache.

    1) This object registers callbacks using the add_condition()
    method.

    2) Prior to serialising the cache object the callbacks are called
    returning a signature dict.

    3) When unpickling the cached object, we call the invalidator to
    produce a signature dict again, and compare this to the pickled
    version.

    The purpose of the callbacks is to represent a signature of the
    current state of execution. If the signature changes, the cache is
    invalidated.
    """
    def __init__(self):
        self.callbacks = {}

    def add_condition(self, key, callback):
        """Callback will be stored under key and should return a string.
        """
        self.callbacks[key] = callback

    def __setstate__(self, state):
        ## We do not actually have any callbacks here - we must use
        ## the global cache invalidator. We cant really get away from
        ## having a global invalidator.
        for k, v in CACHE.invalidator.callbacks.items():
            # TODO: Determine what happens if the state or current callbacks
            # contain a key that's not in the other
            if k in state and v() != state[k]:
                debug.debug("Invaliding cache... {0} (Running) != {1} (Stored) on key {2}".format(v(), state[k], k))

                raise InvalidCache("Running environment inconsistant "
                                   "with pickled environment - "
                                   "invalidating cache.")

    def __getstate__(self):
        """When pickling ourselves we call our callbacks to provide a
        dict of strings (our state signature). This dict should
        reflect all of our running state at the moment. This will then
        be compared to the state signature when unpickling and if its
        different we invalidate the cache.
        """
        result = {}
        for k, v in CACHE.invalidator.callbacks.items():
            result[k] = v()

        debug.debug("Pickling State signature: {0}".format(result))

        return result

class CacheTree(object):
    """ An abstract structure which represents the cache tree """
    def __init__(self, storage = None, cls = CacheNode, invalidator = None):
        self.storage = storage
        self.cls = cls
        self.invalidator = invalidator
        self.root = self.cls('', '', storage = storage, invalidator = invalidator)

    def __getitem__(self, path):
        """Pythonic interface to the cache"""
        return self.check(path, cls = self.cls)

    def invalidate_on(self, key, callback):
        self.invalidator.add_condition(key, callback)

    def check(self, path, callback = None, cls = CacheNode):
        """ Retrieves the node at the path specified """
        # Abort if we haven't been given a location
        if not config.LOCATION:
            return None

        ## Normalise the path
        path = urlparse.urljoin(config.LOCATION + "/", path)

        elements = path.split("/")
        current = self.root

        for e in elements:
            try:
                current = current[e]
            except KeyError:
                if current.stem:
                    next_stem = '/'.join((current.stem, e))
                else:
                    next_stem = e

                payload = None
                if callback is not None:
                    payload = callback()

                node = cls(e, next_stem, storage = self.storage,
                           payload = payload, invalidator = self.invalidator)

                current = node

        return current

class CacheStorage(object):
    """ The base class for implementation storing the cache. """
    ## Characters allowed in filenames (/'s are allowed since we're dealing with URLs only)
    printables = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz-_./"

    def encode(self, string):
        result = ''
        for x in string:
            if x in self.printables:
                result += x
            else:
                result += "%{0:02X}".format(ord(x))

        return result

    def filename(self, url):
        if url.startswith(config.LOCATION):
            # Encode just the path part, since everything else is taken from relatively safe/already used data
            path = self.encode(url[len(config.LOCATION):])
        else:
            raise exceptions.CacheRelativeURLException("Storing non relative URLs is not supported now ({0})".format(url))

        # Join together the bits we need, and abspath it to ensure it's right for the OS it's on
        path = os.path.abspath(os.path.sep.join([config.CACHE_DIRECTORY,
                                                 os.path.basename(config.LOCATION) + ".cache",
                                                 path + '.pickle']))

        return path

    def load(self, url):
        filename = self.filename(url)

        debug.debug("Loading from {0}".format(filename))
        data = open(filename).read()

        debug.trace(level = 3)
        return pickle.loads(data)

    def dump(self, url, payload):
        # TODO: Ensure a better check for ieee1394/non-cachable address spaces than a bad URL
        try:
            filename = self.filename(url)
        except exceptions.CacheRelativeURLException:
            debug.debug("NOT Dumping url {0} - relative URLs are not yet supported".format(url))
            return

        ## Check that the directory exists
        directory = os.path.dirname(filename)
        if not os.access(directory, os.R_OK | os.W_OK | os.X_OK):
            os.makedirs(directory)

        ## Ensure that the payload is flattened - i.e. all generators are converted to lists for pickling
        try:
            data = pickle.dumps(payload)
            debug.debug("Dumping filename {0}".format(filename))
            fd = open(filename, 'w')
            fd.write(data)
            fd.close()
        except (pickle.PickleError, TypeError):
            # Do nothing if the pickle fails
            debug.debug("NOT Dumping filename {0} - contained a non-picklable class".format(filename))

## This is the central cache object
CACHE = CacheTree(CacheStorage(), BlockingNode, invalidator = Invalidator())

def enable_caching(_option, _opt_str, _value, _parser):
    """Turns off caching by replacing the tree with one that only takes BlockingNodes"""
    debug.debug("Enabling Caching")
    # Feels filthy using the global keyword,
    # but I can't figure another way to ensure that
    # the code gets called and overwrites the outer scope
    global CACHE
    CACHE = CacheTree(CacheStorage(), invalidator = Invalidator())
    config.CACHE = True

config.add_option("CACHE", default = False, action = 'callback',
                  cache_invalidator = False,
                  callback = enable_caching,
                  help = "Use caching")

class CacheDecorator(object):
    """ This decorator will memoise a function in the cache """
    def __init__(self, path):
        """Wraps a function in a cache decorator.

        The results of the function will be cached and memoised. Further
        calls to the function will retrieve the result from the
        cache. Cached objects are stored with the specified path as a
        key.

        Args:
           path: Key for storage into the cache. If this is callable,
           it will be called with the function's args and is expected
           to return a string which will be used as a path.

        Returns:
           A decorator.

        Example: Suppose the calculate function is decorated:

        @CacheDecorator(lambda self: "tests/pslist/pid{0}/".format(self._config.PID))
        def calculate(self):
           ....

        Note the use of the callback to finely tune the cache key depending on external variables.
        """
        self.path = path
        self.node = None

    def generate(self, path, g):
        """ Special handling for generators. We pass each iteration
        back immediately, and keep it in a list. Note that if the
        generator is aborted, the cache is not dumped.
        """
        payload = []
        for x in g:
            payload.append(x)
            yield x

        self.dump(path, payload)

    def dump(self, path, payload):
        self.node = CACHE[path]
        self.node.set_payload(payload)
        self.node.dump()

    def _cachewrapper(self, f, s, *args, **kwargs):
        """Wrapper for caching function calls"""
        ## See if the path is callable:
        if callable(self.path):
            path = self.path(s, *args, **kwargs)
        else:
            path = self.path

        ## Check if the result can be retrieved
        self.node = CACHE[path]
        # If this test goes away, we need to change the set_payload exception check
        # to act on dump instead of just the payload
        if self.node:
            payload = self.node.get_payload()
            if payload:
                return payload

        result = f(s, *args, **kwargs)

        ## If the wrapped function is a generator we need to
        ## handle it especially
        if isinstance(result, types.GeneratorType):
            return self.generate(path, result)

        self.dump(path, result)
        return result

    def __call__(self, f):
        def wrapper(s, *args, **kwargs):
            if config.CACHE:
                return self._cachewrapper(f, s, *args, **kwargs)

            return f(s, *args, **kwargs)

        return wrapper

class TestDecorator(CacheDecorator):
    """This decorator is just like a CacheDecorator, but will *always* cache fully"""

    def __call__(self, f):
        def wrapper(s, *args, **kwargs):
            return self._cachewrapper(f, s, *args, **kwargs)
        return wrapper

class Testable(object):
    """ This is a mixin that makes a class response to the unit tests 
    
        It must be inheritted *after* the command class
    """

    def calculate(self):
        """Empty function used to allow mixin"""

    def _flatten(self, item):
        """Flattens an item, including all generators"""
        try:
            # Make sure dicts aren't flattened to lists
            if isinstance(item, dict):
                result = {}
                for i in item:
                    result[self._flatten(i)] = self._flatten(item[i])
                return result

            for x in iter(item):
                flat_x = self._flatten(x)

            return flat_x
        except TypeError:
            return item

    ## This forces the test to be memoised with a key name derived from the class name
    @TestDecorator(lambda self: "tests/unittests/{0}".format(self.__class__.__name__))
    def test(self):
        ## This forces iteration over all keys - this is required in order
        ## to flatten the full list for the cache
        ## We must ensure config.CACHE is False here, otherwise the change isn't registered in this module
        config.CACHE = False
        return self._flatten(self.calculate())



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""

import volatility.plugins.linux.common as linux_common
import volatility.debug as debug
import volatility.obj as obj

class linux_ifconfig(linux_common.AbstractLinuxCommand):
    """Gathers active interfaces"""

    def _get_devs_base(self):
        net_device_ptr = obj.Object("Pointer", offset = self.addr_space.profile.get_symbol("dev_base"), vm = self.addr_space)
        net_device = net_device_ptr.dereference_as("net_device")

        for net_dev in linux_common.walk_internal_list("net_device", "next", net_device):
            yield net_dev

    def _get_devs_namespace(self):
        nslist_addr = self.addr_space.profile.get_symbol("net_namespace_list")
        nethead = obj.Object("list_head", offset = nslist_addr, vm = self.addr_space)

        # walk each network namespace
        # http://www.linuxquestions.org/questions/linux-kernel-70/accessing-ip-address-from-kernel-ver-2-6-31-13-module-815578/
        for net in nethead.list_of_type("net", "list"):

            # walk each device in the current namespace
            for net_dev in net.dev_base_head.list_of_type("net_device", "dev_list"):
                yield net_dev

    def _gather_net_dev_info(self, net_dev):
        mac_addr = net_dev.mac_addr
        promisc  = str(net_dev.promisc)

        in_dev = obj.Object("in_device", offset = net_dev.ip_ptr, vm = self.addr_space)
        
        for dev in in_dev.devices():
            ip_addr = dev.ifa_address.cast('IpAddress')
            name    = dev.ifa_label
            yield (name, ip_addr, mac_addr, promisc)

    def calculate(self):
        linux_common.set_plugin_members(self)

        # newer kernels
        if self.addr_space.profile.get_symbol("net_namespace_list"):
            for net_dev in self._get_devs_namespace():
                for ip_addr_info in self._gather_net_dev_info(net_dev):
                    yield ip_addr_info

        elif self.addr_space.profile.get_symbol("dev_base"):
            for net_dev in self._get_devs_base():
                for ip_addr_info in self._gather_net_dev_info(net_dev):
                    yield ip_addr_info
        
        else:
            debug.error("Unable to determine ifconfig information")
        
    def render_text(self, outfd, data):

        self.table_header(outfd, [("Interface", "16"),
                                  ("IP Address", "20"),
                                  ("MAC Address", "18"),
                                  ("Promiscous Mode", "5")])

        for (name, ip_addr, mac_addr, promisc) in data:
            self.table_row(outfd, name, ip_addr, mac_addr, promisc)

